@@ -1,6 +1,6 @@
 /* block.h
  *
- * Copyright 2002-2004 Vesa Halttunen
+ * Copyright 2002-2005 Vesa Halttunen
  *
  * This file is part of Tutka.
  *
@@ -43,24 +43,21 @@ struct block {
 struct block *block_alloc(int, int, int);
 void block_free(struct block *);
 /* Sets a note in a block */
-void block_set_note(struct block *, unsigned int, unsigned int,
-		   unsigned char, unsigned char, unsigned char);
+void block_set_note(struct block *, unsigned int, unsigned int, unsigned char, unsigned char, unsigned char);
 /* Sets an instrument in a block */
-void block_set_instrument(struct block *, unsigned int, unsigned int,
-			  unsigned char);
+void block_set_instrument(struct block *, unsigned int, unsigned int, unsigned char);
 /* Sets a part of a command in a block */
-void block_set_command(struct block *, unsigned int, unsigned int,
-		       unsigned int, unsigned char, unsigned char);
+void block_set_command(struct block *, unsigned int, unsigned int, unsigned int, unsigned char, unsigned char);
 /* Sets a command in a block */
-void block_set_command_full(struct block *, unsigned int,
-			    unsigned int, unsigned int,
-			    unsigned char, unsigned char);
+void block_set_command_full(struct block *, unsigned int, unsigned int, unsigned int, unsigned char, unsigned char);
 /* Sets the number of tracks in a block */
 void block_set_tracks(struct block *, unsigned int);
 /* Sets the length of a block */
 void block_set_length(struct block *, unsigned int);
 /* Sets the number of command pages in a block */
 void block_set_commandpages(struct block *, unsigned int);
+/* Sets the name of the block */
+void block_set_name(struct block *, char *);
 /* Copies a part of a block to a new block */
 struct block *block_copy(struct block *, int, int, int, int);
 /* Pastes a block to another block in the given position */
