@@ -72,5 +72,7 @@ void on_checkbutton_edit_toggled(GtkToggleButton *, gpointer);
 void on_combobox_keyboard_changed(GtkComboBox *, gpointer);
 void on_combobox_preferences_scheduling_mode_changed(GtkComboBox *, gpointer);
 void on_combobox_instrumentproperties_midiinterface_changed(GtkComboBox *, gpointer);
+void on_button_receive_close_clicked(GtkButton *, gpointer);
+void on_button_receive_stop_clicked(GtkButton *, gpointer);
 
 #endif
