@@ -1,6 +1,6 @@
 /* conversion.h
  *
- * Copyright 2002-2004 Vesa Halttunen
+ * Copyright 2002-2005 Vesa Halttunen
  *
  * This file is part of Tutka.
  *
@@ -23,15 +23,14 @@
 #define _CONVERSION_H
 
 #include "mmd.h"
+#include "smf.h"
 #include "song.h"
 
 /* Converts an MMD2 module to a song */
-struct song *MMD2_convert(struct MMD2 *);
-/* Converts MMD command values at given location */
-void MMD2_commands_convert(unsigned char *, struct MMDMIDICmd3x *);
+struct song *MMD2_convert_song(struct MMD2 *);
 /* Converts a song to an MMD2 module */
-struct MMD2 *song_convert(struct song *);
-/* Converts song command values at given location */
-void song_commands_convert(unsigned char *);
+struct MMD2 *song_convert_MMD2(struct song *);
+/* Converts a song to a standard MIDI file */
+struct smf *song_convert_smf(struct song *);
 
 #endif
