@@ -35,6 +35,11 @@
 
 #define DEFAULT_TEMPO 130
 
+enum {
+  MIDIINTERFACES_AVAILABLE,
+  MIDIINTERFACES_UNAVAILABLE
+};
+
 struct editor {
   /* The song currently being edited */
   struct song *song;
@@ -63,6 +68,12 @@ struct editor {
   /* GConf client for preferences */
   GConfClient *gconfclient;
   guint gconfnotifyid;
+  /* Receive MIDI message status */
+  unsigned char receiveactive;
+  unsigned int receivemessage;
+  unsigned int receivemidiinterface;
+  unsigned int receivedbytes;
+  unsigned char receiveautostop;
 };
 
 /* Initializes the editor */
@@ -149,7 +160,7 @@ unsigned int editor_song_nummessages_get(struct editor *);
 void editor_song_message_insert(struct editor *, unsigned int);
 void editor_song_message_delete(struct editor *, unsigned int);
 void editor_song_message_send(struct editor *, unsigned int);
-void editor_song_message_receive(struct editor *, unsigned int, unsigned char);
+unsigned int editor_song_message_receive(struct editor *);
 void editor_song_section_current_delete(struct editor *);
 unsigned int editor_song_numsections_get(struct editor *);
 void editor_song_section_insert(struct editor *, unsigned int);
@@ -213,9 +224,9 @@ void editor_scheduler_set(struct editor *, unsigned int);
 /* Returns the scheduling mode */
 unsigned int editor_scheduler_get(struct editor *);
 /* Sets the name of the MIDI interface to use */
-void editor_midi_interface_set(struct editor *, char *, unsigned int);
+void editor_midi_interface_set(struct editor *, char *, unsigned int, unsigned int);
 /* Returns the name of the MIDI interface to use */
-char *editor_midi_interface_get(struct editor *, unsigned int);
+char *editor_midi_interface_get(struct editor *, unsigned int, unsigned int);
 /* Marks the enabled interfaces as enabled in the preferences */
 void editor_midi_interface_set_from_current(struct editor *);
 /* Enables or disables a MIDI interface by name */
