@@ -35,9 +35,8 @@ struct gui {
   GtkWidget *window_about;
   unsigned int *numplayseqs, *numblocks;
 
-  /* FIX: These should not be here. Current Message number */
+  /* FIX: This should not be here. Current Message number */
   int messagelist_message;
-  unsigned char messagelist_autostop;
 
   unsigned int refresh_mask, callback_refresh_mask;
   struct block *refresh_tracker_block;
@@ -155,7 +154,8 @@ void gui_trackvolumes_refresh(struct gui *);
 void gui_preferences_refresh(struct gui *);
 void gui_tracker_refresh(struct gui *);
 void gui_song_refresh(struct gui *);
-
+void gui_receive_refresh(struct gui *, unsigned int, unsigned int, gboolean);
+  
 Tracker *gui_get_tracker(struct gui *);
 int gui_get_tracker_position(struct gui *);
 
