@@ -1,6 +1,6 @@
 /* player.h
  *
- * Copyright 2002-2004 Vesa Halttunen
+ * Copyright 2002-2005 Vesa Halttunen
  *
  * This file is part of Tutka.
  *
@@ -34,26 +34,30 @@ enum {
   MODE_PLAY_BLOCK
 };
 
+struct player_interface_status {
+  char midichannel;
+  char volume;
+  char note;
+  char hold;
+};
+
 /* Track status values */
-enum {
-  TRACK_STATUS_INSTRUMENT,
-  TRACK_STATUS_MIDI_CHANNEL,
-  TRACK_STATUS_NOTE,
-  TRACK_STATUS_VOLUME,
-  TRACK_STATUS_HOLD,
-  TRACK_STATUS_PREVIOUS_COMMAND,
-  TRACK_STATUS_NUMVALUES
+struct player_track_status {
+  char instrument;
+  char previouscommand;
+  struct player_interface_status **interfaces;
 };
 
 /* 128 MIDI controllers plus aftertouch, channel pressure and pitch wheel */
 enum {
-  VALUES=(128+3),
-  VALUES_AFTERTOUCH=128,
-  VALUES_CHANNEL_PRESSURE=129,
-  VALUES_PITCH_WHEEL=130
+  VALUES = (128 + 3),
+  VALUES_AFTERTOUCH = 128,
+  VALUES_CHANNEL_PRESSURE = 129,
+  VALUES_PITCH_WHEEL = 130
 };
 
 enum {
+  SCHED_NONE,
   SCHED_RTC,
   SCHED_NANOSLEEP
 };
@@ -68,27 +72,35 @@ struct player {
   /* Player scheduling mode */
   unsigned int sched;
   /* Status of tracks; notes playing */
-  char *trackstatus;
+  struct player_track_status **trackstatus;
   /* MIDI controller values; one for each controller on each channel */
-  unsigned char midicontrollervalues[16*VALUES];
+  unsigned char **midicontrollervalues;
   /* For measuring how long the song has been playing */
   struct timeval playingstarted, playedsofar;
+  /* Ticks passed after playing started */
+  unsigned int tickssofar;
   /* Player thread pointer */
   GThread *thread;
   /* Mutex for the player thread */
   GMutex *mutex;
   /* Kill player flag (the mutex must be used when accessing) */
   unsigned int killthread;
-  /* MIDI data structure */
-  struct midi *midi;
+  /* MIDI data structures */
+  struct midi **midiinterfaces;
+  /* Number of MIDI data structures */
+  unsigned int nummidiinterfaces;
   /* RTC device file descriptor */
   int rtc;
   /* Editor for this player */
   struct editor *editor;
+  /* Indicates whether some tracks are soloed or not */
+  unsigned int solo;
 };
 
+/* Plays a song to a MIDI interface witout any scheduling (for export) */
+void player_midi_export(struct song *, struct midi *);
 /* Creates a player for a song */
-struct player *player_open(struct song *, struct editor *);
+struct player *player_open(struct song *, struct editor *, struct midi **, unsigned int, unsigned int);
 /* Closes a player for a song */
 void player_close(struct player *);
 
@@ -98,8 +110,7 @@ void player_start(struct player *, unsigned int, int, int, int, int);
 void player_stop(struct player *);
 
 /* Plays a note using given instrument on a given channel */
-void player_play_note(struct player *, unsigned int, unsigned char,
-		      unsigned char, unsigned char);
+void player_play_note(struct player *, unsigned int, unsigned char, unsigned char, unsigned char);
 /* Stops notes playing at the moment */
 void player_stop_notes(struct player *);
 /* Stops all notes */
@@ -117,5 +128,10 @@ void player_set_playseq(struct player *, int);
 void player_set_position(struct player *, int);
 void player_set_block(struct player *, int);
 void player_set_line(struct player *, int);
+  
+/* Sets whether some tracks are considered soloed or not */
+void player_set_solo(struct player *, unsigned int);
+/* Checks whether some tracks are soloed or not */
+void player_check_solo(struct player *);
 
 #endif
