@@ -45,7 +45,7 @@ void on_menuitem_file_open_activate(GtkMenuItem * menuitem, gpointer user_data)
 {
   struct gui *gui = (struct gui *)user_data;
   GtkWidget *fs = glade_xml_get_widget(gui->xml, "filechooserdialog_file_open");
-  
+
   if (!GTK_WIDGET_VISIBLE(fs))
     gtk_widget_show_all(fs);
 
@@ -102,7 +102,7 @@ void on_menuitem_edit_cut_activate(GtkMenuItem * menuitem, gpointer user_data)
 
   if (tracker_is_in_selection_mode(tracker))
     tracker_mark_selection(tracker, FALSE);
-  
+
   editor_song_block_current_copyarea(gui->editor, tracker->sel_start_ch, tracker->sel_start_row, tracker->sel_end_ch, tracker->sel_end_row);
   editor_song_block_current_clear(gui->editor, tracker->sel_start_ch, tracker->sel_start_row, tracker->sel_end_ch, tracker->sel_end_row);
   tracker_clear_mark_selection(tracker);
@@ -191,7 +191,7 @@ void on_menuitem_edit_expandshrink_activate(GtkMenuItem * menuitem, gpointer use
 void on_scale_tempo_value_changed(GtkRange * range, gpointer user_data)
 {
   struct gui *gui = (struct gui *)user_data;
-    
+
   editor_song_tempo_set(gui->editor, (unsigned int)gtk_range_get_value(range));
   editor_set_tempo(gui->editor);
 }
@@ -285,10 +285,10 @@ void on_menuitem_song_properties_activate(GtkMenuItem * menuitem, gpointer user_
 void on_menuitem_block_cut_activate(GtkMenuItem * menuitem, gpointer user_data)
 {
   struct gui *gui = (struct gui *)user_data;
-  
+
   editor_song_block_current_copyblock(gui->editor);
   editor_song_block_current_clear(gui->editor, 0, 0, editor_song_block_current_tracks_get(gui->editor) - 1, editor_song_block_current_length_get(gui->editor) - 1);
-  tracker_redraw((Tracker *)glade_xml_get_widget(gui->xml, "tracker"));
+  tracker_redraw((Tracker *) glade_xml_get_widget(gui->xml, "tracker"));
 }
 
 void on_menuitem_block_copy_activate(GtkMenuItem * menuitem, gpointer user_data)
@@ -303,7 +303,7 @@ void on_menuitem_block_paste_activate(GtkMenuItem * menuitem, gpointer user_data
   struct gui *gui = (struct gui *)user_data;
 
   editor_song_block_current_copyblock_paste(gui->editor);
-  tracker_redraw((Tracker *)glade_xml_get_widget(gui->xml, "tracker"));
+  tracker_redraw((Tracker *) glade_xml_get_widget(gui->xml, "tracker"));
 }
 
 void on_menuitem_block_selectall_activate(GtkMenuItem * menuitem, gpointer user_data)
@@ -499,7 +499,7 @@ void on_menuitem_midi_killallnotes_activate(GtkMenuItem * menuitem, gpointer use
 void on_menuitem_midi_resetpitch_activate(GtkMenuItem * menuitem, gpointer user_data)
 {
   struct gui *gui = (struct gui *)user_data;
-    
+
   editor_player_pitch_reset(gui->editor);
 }
 
@@ -726,6 +726,11 @@ void on_button_message_appendnew_clicked(GtkButton * button, gpointer user_data)
   struct gui *gui = (struct gui *)user_data;
 
   editor_song_message_insert(gui->editor, editor_song_nummessages_get(gui->editor));
+
+  /* If no message was selected select the new message */
+  if (gui->messagelist_message == -1)
+    gui->messagelist_message = editor_song_nummessages_get(gui->editor) - 1;
+
   gui_messagelist_refresh(gui, TRUE);
 }
 
@@ -735,15 +740,15 @@ void on_button_message_delete_clicked(GtkButton * button, gpointer user_data)
 
   if (gui->messagelist_message != -1) {
     int n = editor_song_nummessages_get(gui->editor);
-    
+
     editor_song_message_delete(gui->editor, gui->messagelist_message);
 
+    n--;
     if (gui->messagelist_message >= n)
       gui->messagelist_message = n - 1;
 
     gui_messagelist_refresh(gui, TRUE);
   }
-
 }
 
 void on_button_message_insertnew_clicked(GtkButton * button, gpointer user_data)
@@ -752,6 +757,7 @@ void on_button_message_insertnew_clicked(GtkButton * button, gpointer user_data)
 
   editor_song_message_insert(gui->editor, gui->messagelist_message);
 
+  /* If no message was selected select the new message */
   if (gui->messagelist_message == -1)
     gui->messagelist_message = 0;
 
@@ -769,20 +775,83 @@ void on_button_message_send_clicked(GtkButton * button, gpointer user_data)
 void on_button_message_receive_clicked(GtkButton * button, gpointer user_data)
 {
   struct gui *gui = (struct gui *)user_data;
-  if (gui->messagelist_message != -1) {
-    editor_song_message_receive(gui->editor, gui->messagelist_message, gui->messagelist_autostop);
-    gui_messagelist_refresh(gui, FALSE);
+  GtkWidget *window = glade_xml_get_widget(gui->xml, "dialog_receive");
+
+  if (!GTK_WIDGET_VISIBLE(window))
+    gtk_widget_show(window);
+
+  gui->editor->receivemessage = gui->messagelist_message;
+  
+  gtk_window_present(GTK_WINDOW(window));
+  gui_receive_refresh(gui, 0, 0, TRUE);
+}
+
+void on_button_receive_clicked(GtkButton * button, gpointer user_data)
+{
+  struct gui *gui = (struct gui *)user_data;
+  GtkWindow *window = GTK_WINDOW(glade_xml_get_widget(gui->xml, "dialog_receive"));
+
+  if (!gui->editor->receiveactive && gui->editor->receivemidiinterface != -1) {
+    /* Start receiving */
+    gtk_window_set_modal(window, TRUE);
+    gtk_widget_set_sensitive(glade_xml_get_widget(gui->xml, "combobox_receive"), FALSE);
+    gtk_widget_set_sensitive(glade_xml_get_widget(gui->xml, "checkbutton_receive_autostop"), FALSE);
+    gtk_widget_set_sensitive(glade_xml_get_widget(gui->xml, "button_receive"), FALSE);
+    gtk_widget_set_sensitive(glade_xml_get_widget(gui->xml, "button_receive_close"), FALSE);
+    gtk_widget_set_sensitive(glade_xml_get_widget(gui->xml, "button_receive_stop"), TRUE);
+
+    /* Reset the progress bar */
+    gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(glade_xml_get_widget(gui->xml, "progressbar_receive")), 0);
+
+    gui->editor->receivedbytes = 0;
+    gui->editor->receiveactive = 1;
   }
 }
 
+void on_button_receive_stop_clicked(GtkButton * button, gpointer user_data)
+{
+  struct gui *gui = (struct gui *)user_data;
+  GtkWindow *window = GTK_WINDOW(glade_xml_get_widget(gui->xml, "dialog_receive"));
+
+  if (gui->editor->receiveactive) {
+    /* Stop receiving */
+    gtk_window_set_modal(window, FALSE);
+    gtk_widget_set_sensitive(glade_xml_get_widget(gui->xml, "combobox_receive"), TRUE);
+    gtk_widget_set_sensitive(glade_xml_get_widget(gui->xml, "checkbutton_receive_autostop"), TRUE);
+    gtk_widget_set_sensitive(glade_xml_get_widget(gui->xml, "button_receive"), TRUE);
+    gtk_widget_set_sensitive(glade_xml_get_widget(gui->xml, "button_receive_close"), TRUE);
+    gtk_widget_set_sensitive(glade_xml_get_widget(gui->xml, "button_receive_stop"), FALSE);
+
+    /* Reset the progress bar */
+    gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(glade_xml_get_widget(gui->xml, "progressbar_receive")), 0);
+
+    /* Remove unnecessary bytes if necessary */
+    if (gui->editor->receivedbytes < editor_song_message_length_get(gui->editor, gui->editor->receivemessage))
+      editor_song_message_length_set(gui->editor, gui->editor->receivemessage, gui->editor->receivedbytes);
+
+    gui_messagelist_refresh(gui, TRUE);
+
+    gui->editor->receiveactive = 0;
+  }
+}
+
+void on_button_receive_close_clicked(GtkButton * button, gpointer user_data)
+{
+  struct gui *gui = (struct gui *)user_data;
+  GtkWidget *window = glade_xml_get_widget(gui->xml, "dialog_receive");
+
+  if (GTK_WIDGET_VISIBLE(window))
+    gtk_widget_hide(window);
+}
+
 void on_checkbutton_message_autostop_toggled(GtkToggleButton * togglebutton, gpointer user_data)
 {
   struct gui *gui = (struct gui *)user_data;
 
   if (gtk_toggle_button_get_active(togglebutton))
-    gui->messagelist_autostop = 1;
+    gui->editor->receiveautostop = 1;
   else
-    gui->messagelist_autostop = 0;
+    gui->editor->receiveautostop = 0;
 
 }
 
@@ -866,7 +935,7 @@ void on_selection_playseq_changed(GtkTreeSelection * selection, gpointer user_da
 void on_button_playseq_delete_clicked(GtkButton * button, gpointer user_data)
 {
   struct gui *gui = (struct gui *)user_data;
-    
+
   editor_song_playseq_current_delete_current(gui->editor);
   gui_playseq_refresh(gui, TRUE);
   gui_info_refresh(gui);
@@ -884,7 +953,7 @@ void on_button_playseq_append_clicked(GtkButton * button, gpointer user_data)
 void on_button_playseq_insert_clicked(GtkButton * button, gpointer user_data)
 {
   struct gui *gui = (struct gui *)user_data;
-    
+
   editor_song_playseq_current_insert(gui->editor, gui->editor->position);
   gui_playseq_refresh(gui, TRUE);
   gui_info_refresh(gui);
@@ -893,7 +962,7 @@ void on_button_playseq_insert_clicked(GtkButton * button, gpointer user_data)
 void on_button_playseqlist_insertnew_clicked(GtkButton * button, gpointer user_data)
 {
   struct gui *gui = (struct gui *)user_data;
-    
+
   editor_song_playseq_insert(gui->editor, gui->editor->playseq);
   gui_info_refresh(gui);
   gui_playseqlist_refresh(gui, TRUE);
@@ -904,7 +973,7 @@ void on_button_playseqlist_insertnew_clicked(GtkButton * button, gpointer user_d
 void on_button_playseqlist_appendnew_clicked(GtkButton * button, gpointer user_data)
 {
   struct gui *gui = (struct gui *)user_data;
-    
+
   editor_song_playseq_insert(gui->editor, editor_song_numplayseqs_get(gui->editor));
   gui_info_refresh(gui);
   gui_playseqlist_refresh(gui, TRUE);
@@ -941,7 +1010,7 @@ void on_selection_blocklist_changed(GtkTreeSelection * selection, gpointer user_
 void on_button_blocklist_insertnew_clicked(GtkButton * button, gpointer user_data)
 {
   struct gui *gui = (struct gui *)user_data;
-    
+
   editor_song_block_insert(gui->editor, gui->editor->block, gui->editor->block);
   gui_tracker_refresh(gui);
   gui_blocklist_refresh(gui, TRUE);
@@ -952,7 +1021,7 @@ void on_button_blocklist_insertnew_clicked(GtkButton * button, gpointer user_dat
 void on_button_blocklist_appendnew_clicked(GtkButton * button, gpointer user_data)
 {
   struct gui *gui = (struct gui *)user_data;
-    
+
   editor_song_block_insert(gui->editor, editor_song_numblocks_get(gui->editor), gui->editor->block);
   gui_blocklist_refresh(gui, TRUE);
   gui_playseq_refresh(gui, TRUE);
@@ -974,7 +1043,7 @@ void on_button_blocklist_delete_clicked(GtkButton * button, gpointer user_data)
   gui_tracker_refresh(gui);
 }
 
-gboolean on_entry_instrumentproperties_name_changed(GtkEditable *widget, gpointer user_data)
+gboolean on_entry_instrumentproperties_name_changed(GtkEditable * widget, gpointer user_data)
 {
   struct gui *gui = (struct gui *)user_data;
 
@@ -997,7 +1066,7 @@ void on_button_file_saveas_ok_clicked(GtkButton * button, gpointer user_data)
 {
   struct gui *gui = (struct gui *)user_data;
   GtkWidget *fs = glade_xml_get_widget(gui->xml, "filechooserdialog_file_saveas");
-  
+
   editor_song_save(gui->editor, (char *)gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(fs)));
   gtk_widget_hide(fs);
 }
@@ -1029,10 +1098,10 @@ void on_button_file_saveas_cancel_clicked(GtkButton * button, gpointer user_data
     gtk_widget_hide(fs);
 }
 
-gboolean on_entry_songname_changed(GtkEditable *widget, gpointer user_data)
+gboolean on_entry_songname_changed(GtkEditable * widget, gpointer user_data)
 {
   struct gui *gui = (struct gui *)user_data;
-    
+
   editor_song_name_set(gui->editor, (char *)gtk_entry_get_text(GTK_ENTRY(widget)));
   gui_window_title_refresh(gui);
 
@@ -1063,11 +1132,51 @@ void on_spinbutton_instrument_changed(GtkEditable * editable, gpointer user_data
   gtk_widget_grab_focus(glade_xml_get_widget(gui->xml, "button_stop"));
 }
 
+gint on_spinbutton_instrument_output(GtkSpinButton * spin_button, gpointer user_data)
+{
+  gchar v[2];
+  int i = (int)spin_button->adjustment->value;
+
+  /* Convert the instrument number to 0-9 / A-Z */
+  if (i < 10)
+    v[0] = '0' + i;
+  else
+    v[0] = 'A' + (i - 10);
+  v[1] = 0;
+
+  /* Only update the GtkEntry if the value was changed */
+  if (strcmp(v, gtk_entry_get_text(GTK_ENTRY(spin_button))))
+    gtk_entry_set_text(GTK_ENTRY(spin_button), v);
+
+  return TRUE;
+}
+
+gint on_spinbutton_instrument_input(GtkSpinButton * spin_button, gpointer arg1, gpointer user_data)
+{
+  const gchar *c = gtk_entry_get_text(GTK_ENTRY(spin_button));
+  gdouble *new_val = arg1;
+
+  if (c == NULL || strlen(c) > 1)
+    return GTK_INPUT_ERROR;
+
+  /* Convert the 0-9 / A-Z string to an integer */
+  if (c[0] >= '0' && c[0] <= '9')
+    *new_val = (gdouble) (c[0] - '0');
+  else if (c[0] >= 'A' && c[0] <= 'Z')
+    *new_val = (gdouble) ((c[0] - 'A') + 10);
+  else if (c[0] >= 'a' && c[0] <= 'z')
+    *new_val = (gdouble) ((c[0] - 'a') + 10);
+  else
+    return GTK_INPUT_ERROR;
+
+  return TRUE;
+}
+
 void on_adjustment_trackvolumes_volume_changed(GtkAdjustment * adjustment, gpointer user_data)
 {
   struct gui *gui = (struct gui *)user_data;
   int track = (int)g_object_get_data(G_OBJECT(adjustment), "track");
-  
+
   editor_song_track_volume_set(gui->editor, track, (unsigned char)adjustment->value);
 }
 
@@ -1093,7 +1202,7 @@ void on_togglebutton_trackvolumes_solo_toggled(GtkToggleButton * togglebutton, g
     editor_song_track_solo_set(gui->editor, track, 0);
 }
 
-void on_entry_trackvolumes_name_changed(GtkEditable *widget, gpointer user_data)
+void on_entry_trackvolumes_name_changed(GtkEditable * widget, gpointer user_data)
 {
   struct gui *gui = (struct gui *)user_data;
   int track = (int)g_object_get_data(G_OBJECT(widget), "track");
@@ -1101,7 +1210,7 @@ void on_entry_trackvolumes_name_changed(GtkEditable *widget, gpointer user_data)
   editor_song_track_name_set(gui->editor, track, (char *)gtk_entry_get_text(GTK_ENTRY(widget)));
 }
 
-void on_combobox_keyboard_changed(GtkComboBox *combobox, gpointer user_data)
+void on_combobox_keyboard_changed(GtkComboBox * combobox, gpointer user_data)
 {
   struct gui *gui = (struct gui *)user_data;
 
@@ -1345,7 +1454,7 @@ void on_button_trackvolumes_mute_clicked(GtkButton * button, gpointer user_data)
   gui_trackvolumes_refresh(gui);
 }
 
-void on_combobox_preferences_scheduling_mode_changed(GtkComboBox *widget, gpointer user_data)
+void on_combobox_preferences_scheduling_mode_changed(GtkComboBox * widget, gpointer user_data)
 {
   struct gui *gui = (struct gui *)user_data;
 
@@ -1394,7 +1503,7 @@ void on_button_message_save_ok_clicked(GtkButton * button, gpointer user_data)
 {
   struct gui *gui = (struct gui *)user_data;
   GtkWidget *fs = glade_xml_get_widget(gui->xml, "filechooserdialog_message_save");
-  
+
   editor_song_message_save(gui->editor, gui->messagelist_message, (char *)gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(fs)));
 
   /* Close the file selector and refresh the GUI */
@@ -1642,7 +1751,7 @@ void on_cell_sectionlist_edited(GtkCellRendererText * cell, gchar * path_string,
       if (*error == 0 && value > 0 && value <= editor_song_numplayseqs_get(gui->editor)) {
 	gtk_list_store_set(GTK_LIST_STORE(model), &iter, column, value, -1);
 	gtk_list_store_set(GTK_LIST_STORE(model), &iter, column + 1, editor_song_playseq_name_get(gui->editor, value - 1), -1);
-        editor_song_section_current_set(gui->editor, value - 1);
+	editor_song_section_current_set(gui->editor, value - 1);
       }
       break;
     }
@@ -1713,7 +1822,7 @@ void on_cell_playseq_edited(GtkCellRendererText * cell, gchar * path_string, gch
       /* Only update if the entered value was valid */
       if (*error == 0 && value > 0 && value <= editor_song_numblocks_get(gui->editor)) {
 	gtk_list_store_set(GTK_LIST_STORE(model), &iter, column, value, -1);
-        editor_song_playseq_current_set(gui->editor, value - 1);
+	editor_song_playseq_current_set(gui->editor, value - 1);
 	gui_playseq_refresh(gui, TRUE);
       }
       break;
@@ -1802,7 +1911,7 @@ void on_cell_blocklist_edited(GtkCellRendererText * cell, gchar * path_string, g
     {
       editor_song_block_current_commandpages_set(gui->editor, value);
       if (gui->editor->cmdpage >= editor_song_block_current_commandpages_get(gui->editor)) {
-        gui->editor->cmdpage = editor_song_block_current_commandpages_get(gui->editor) - 1;
+	gui->editor->cmdpage = editor_song_block_current_commandpages_get(gui->editor) - 1;
 	gui_tracker_refresh(gui);
       }
       gui_info_refresh(gui);
@@ -1849,8 +1958,8 @@ void on_cell_messagelist_edited(GtkCellRendererText * cell, gchar * path_string,
       value = (int)strtol(new_text, &error, 10);
       /* Only update if the entered value was valid */
       if (*error == 0 && gui->messagelist_message != -1) {
-        gtk_list_store_set(GTK_LIST_STORE(model), &iter, column, value, -1);
-        editor_song_message_length_set(gui->editor, gui->messagelist_message, value);
+	gtk_list_store_set(GTK_LIST_STORE(model), &iter, column, value, -1);
+	editor_song_message_length_set(gui->editor, gui->messagelist_message, value);
       }
       break;
     }
@@ -1863,21 +1972,25 @@ void on_cell_messagelist_toggled(GtkCellRendererToggle * cell, gchar * path_stri
   GtkTreeView *treeview;
   GtkTreeModel *model;
   GtkTreeIter iter;
+  GValue message = { 0 };
+  unsigned int i;
 
   /* Get the iterator */
   treeview = GTK_TREE_VIEW(glade_xml_get_widget(gui->xml, "treeview_messagelist"));
   model = gtk_tree_view_get_model(treeview);
   gtk_tree_model_get_iter_from_string(model, &iter, path_string);
 
+  /* The selected message may be changed - get message number */
+  gtk_tree_model_get_value(model, &iter, GUI_MESSAGELIST_COLUMN_NUMBER, &message);
+  gui->messagelist_message = g_value_get_int(&message);
+
   /* Update the model */
-  if (gui->messagelist_message != -1) {
-    unsigned int i = editor_song_message_autosend_get(gui->editor, gui->messagelist_message) ^ 1;
-    editor_song_message_autosend_set(gui->editor, gui->messagelist_message, i);   
-    gtk_list_store_set(GTK_LIST_STORE(model), &iter, GUI_MESSAGELIST_COLUMN_AUTOSEND, i, -1);
-  }
+  i = editor_song_message_autosend_get(gui->editor, gui->messagelist_message) ^ 1;
+  editor_song_message_autosend_set(gui->editor, gui->messagelist_message, i);
+  gtk_list_store_set(GTK_LIST_STORE(model), &iter, GUI_MESSAGELIST_COLUMN_AUTOSEND, i, -1);
 }
 
-void on_cell_preferences_midi_interfaces_toggled(GtkCellRendererToggle *cell, gchar *path_string, gpointer user_data)
+void on_cell_preferences_midi_interfaces_toggled(GtkCellRendererToggle * cell, gchar * path_string, gpointer user_data)
 {
   struct gui *gui = (struct gui *)user_data;
   GtkTreeView *treeview;
@@ -1887,7 +2000,7 @@ void on_cell_preferences_midi_interfaces_toggled(GtkCellRendererToggle *cell, gc
   gboolean state;
   GValue device = { 0 };
   unsigned int direction;
-  
+
   /* Column number is passed as renderer object data */
   gpointer columnptr = g_object_get_data(G_OBJECT(cell), "column");
 
@@ -1906,29 +2019,49 @@ void on_cell_preferences_midi_interfaces_toggled(GtkCellRendererToggle *cell, gc
   treeview = GTK_TREE_VIEW(glade_xml_get_widget(gui->xml, "treeview_preferences_midi_interfaces"));
   model = gtk_tree_view_get_model(treeview);
   gtk_tree_model_get_iter_from_string(model, &iter, path_string);
-  
+
   /* Try changing the state of a device */
   g_object_get(cell, "active", &state, NULL);
   if (state)
     state = FALSE;
   else
     state = TRUE;
-  
+
   gtk_tree_model_get_value(model, &iter, GUI_PREFERENCES_MIDI_INTERFACES_COLUMN_DEVICENAME, &device);
-  if(editor_midi_interface_status_set(gui->editor, (char *)g_value_get_string(&device), direction, state))
+  if (editor_midi_interface_status_set(gui->editor, (char *)g_value_get_string(&device), direction, state))
     state = TRUE;
   else
     state = FALSE;
 
   /* Update the model according to the true status of the device */
   gtk_list_store_set(GTK_LIST_STORE(model), &iter, column, state, -1);
-  
+
   g_value_unset(&device);
 }
 
-void on_combobox_instrumentproperties_midiinterface_changed(GtkComboBox *combobox, gpointer user_data)
+void on_combobox_instrumentproperties_midiinterface_changed(GtkComboBox * combobox, gpointer user_data)
 {
   struct gui *gui = (struct gui *)user_data;
 
   editor_song_instrument_current_midiinterface_set(gui->editor, (unsigned short)gtk_combo_box_get_active(combobox));
 }
+
+void on_dialog_receive_delete_event(GtkWidget * widget, GdkEvent * event, gpointer user_data)
+{
+  on_button_receive_close_clicked(NULL, user_data);
+}
+
+void on_combobox_receive_changed(GtkComboBox * combobox, gpointer user_data)
+{
+  struct gui *gui = (struct gui *)user_data;
+  GtkTreeModel *model;
+  GtkTreeIter iter;
+  GValue interface = { 0 };
+
+  if (gtk_combo_box_get_active_iter(combobox, &iter)) {
+    model = gtk_combo_box_get_model(combobox);
+    gtk_tree_model_get_value(model, &iter, 1, &interface);
+    gui->editor->receivemidiinterface = g_value_get_int(&interface);
+    g_value_unset(&interface);
+  }
+}
