@@ -1,6 +1,6 @@
 /* conversion.c
  *
- * Copyright 2002-2004 Vesa Halttunen
+ * Copyright 2002-2005 Vesa Halttunen
  *
  * This file is part of Tutka.
  *
@@ -21,20 +21,113 @@
 
 #include <string.h>
 #include <glib.h>
+#include "tutka.h"
+#include "midi.h"
+#include "player.h"
 #include "conversion.h"
 
-/* Converts an MMD2 module to a song */
-struct song *MMD2_convert(struct MMD2 *mmd)
+/* Converts MMD command values at given location */
+static void MMD2_commands_convert(unsigned char *command, struct MMDMIDICmd3x *mmdcmd3x)
 {
+  switch (command[0]) {
+  case COMMAND_PREVIOUS_COMMAND_VALUE:
+    if (command[1] != 0) {
+      command[0] = 0x80;
+      command[1] &= 0x7f;
+    }
+    break;
+  case COMMAND_VELOCITY:
+    /* Fix volume range to 0..127 */
+    if (command[1] == 64)
+      command[1] = 127;
+    else
+      command[1] *= 2;
+    break;
+  case COMMAND_DELAY:
+    /* Command 1F in MMD is both note delay and retrigger */
+    if (command[1] > 15)
+      command[1] = command[1] >> 4;
+    else
+      command[0] = COMMAND_RETRIGGER;
+    break;
+  case COMMAND_TEMPO:
+    /* Values >0xf0 have special meanings in OctaMED */
+    if (command[1] > 0xf0) {
+      switch (command[1]) {
+      case 0xf1:
+	/* Makes a single note play twice */
+	command[0] = COMMAND_RETRIGGER;
+	command[1] = 3;
+	break;
+      case 0xf2:
+	/* Delays the start of a note by half a line */
+	command[0] = COMMAND_DELAY;
+	command[1] = 3;
+	break;
+      case 0xf3:
+	/* Works like 0FF1 except the note is played three times */
+	command[0] = COMMAND_RETRIGGER;
+	command[1] = 2;
+	break;
+      case 0xf4:
+	/* Delays the note one-third of a line */
+	command[0] = COMMAND_DELAY;
+	command[1] = 2;
+	break;
+      case 0xf5:
+	/* Delays the note two thirds of a line */
+	command[0] = COMMAND_DELAY;
+	command[1] = 4;
+	break;
+      case 0xfe:
+	/* Stops the song playing */
+	command[0] = COMMAND_TEMPO;
+	command[1] = 0;
+	break;
+      case 0xff:
+	/* Stops the note on the current track */
+	command[0] = COMMAND_VELOCITY;
+	command[1] = 0;
+	break;
+      default:
+	command[0] = 0;
+	command[1] = 0;
+	break;
+      }
+    }
+    break;
+  case 0x31:
+  case 0x32:
+  case 0x33:
+  case 0x34:
+  case 0x35:
+  case 0x36:
+  case 0x37:
+  case 0x38:
+  case 0x39:
+  case 0x3a:
+  case 0x3b:
+  case 0x3c:
+  case 0x3d:
+  case 0x3e:
+  case 0x3f:
+    if (mmdcmd3x->ctrlr_types[command[0] - 0x31] == MCS_TYPE_STD_MSB)
+      command[0] = mmdcmd3x->ctrlr_numbers[command[0] - 0x31] + 0x80;
+    break;
+  default:
+    break;
+  }
+}
+
+/* Converts an MMD2 module to a song */
+struct song *MMD2_convert_song(struct MMD2 *mmd)
+{
+  nullcheck_pointer(mmd, MMD2_convert_song);
+
   struct song *song;
   int i, j, k, l;
   unsigned char commandconversion[256];
 
-  if (mmd == NULL) {
-    fprintf(stderr, "MMD2_convert() called with null MMD2\n");
-    return NULL;
-  }
-
   /* Create a table for converting MMD commands; values need special handling */
   for (i = 0; i < 256; i++)
     commandconversion[i] = i;
@@ -158,26 +251,27 @@ struct song *MMD2_convert(struct MMD2 *mmd)
       if (mmd->expdata) {
 	gchar *name = g_convert(mmd->expdata->iinfo[i].name, strlen(mmd->expdata->iinfo[i].name), "UTF-8", "ISO-8859-1", NULL, NULL, NULL);
 	song->instruments[i]->name = strdup(name);
-	song->instruments[i]->hold = mmd->expdata->exp_smp[i].hold;
+	song->instruments[i]->outputs[0]->hold = mmd->expdata->exp_smp[i].hold;
 	g_free(name);
       }
-      song->instruments[i]->midichannel = mmd->song->sample[i].midich - 1;
-      song->instruments[i]->midipreset = mmd->song->sample[i].midipreset;
-      song->instruments[i]->transpose = mmd->song->sample[i].strans;
+      song->instruments[i]->outputs[0]->midichannel = mmd->song->sample[i].midich - 1;
+      song->instruments[i]->outputs[0]->midipreset = mmd->song->sample[i].midipreset;
+      song->instruments[i]->outputs[0]->transpose = mmd->song->sample[i].strans;
       if (mmd->song->sample[i].svol == 64)
-	song->instruments[i]->defaultvelocity = 127;
+	song->instruments[i]->outputs[0]->defaultvelocity = 127;
       else
-	song->instruments[i]->defaultvelocity = mmd->song->sample[i].svol * 2;
+	song->instruments[i]->outputs[0]->defaultvelocity = mmd->song->sample[i].svol * 2;
     }
 
     /* Track volumes */
     song->maxtracks = mmd->song->numtracks;
-    song->trackvolumes = (unsigned int *)calloc(song->maxtracks, sizeof(unsigned int));
+    song->tracks = (struct track **)calloc(song->maxtracks, sizeof(struct track *));
     for (i = 0; i < song->maxtracks; i++) {
+      song->tracks[i] = (struct track *)calloc(1, sizeof(struct track));
       if (mmd->song->trackvols[i] < 64)
-	song->trackvolumes[i] = mmd->song->trackvols[i] * 2;
+	song->tracks[i]->volume = mmd->song->trackvols[i] * 2;
       else
-	song->trackvolumes[i] = 127;
+	song->tracks[i]->volume = 127;
     }
     if (mmd->song->mastervol < 64)
       song->mastervolume = mmd->song->mastervol * 2;
@@ -214,75 +308,18 @@ struct song *MMD2_convert(struct MMD2 *mmd)
   return song;
 }
 
-/* Converts MMD command values at given location */
-void MMD2_commands_convert(unsigned char *command, struct MMDMIDICmd3x *mmdcmd3x)
+/* Converts song command values at given location */
+static void song_commands_convert(unsigned char *command)
 {
   switch (command[0]) {
-  case COMMAND_PREVIOUS_COMMAND_VALUE:
-    if (command[1] != 0) {
-      command[0] = 0x80;
-      command[1] &= 0x7f;
-    }
+  case 0x0c:
+    /* Fix volume range to 0..64 */
+    command[1] = (command[1] + 1) / 2;
     break;
-  case COMMAND_VELOCITY:
-    /* Fix volume range to 0..127 */
-    if (command[1] == 64)
-      command[1] = 127;
-    else
-      command[1] *= 2;
-    break;
-  case COMMAND_DELAY:
-    /* Command 1F in MMD is both note delay and retrigger */
-    if (command[1] > 15)
-      command[1] = command[1] >> 4;
-    else
-      command[0] = COMMAND_RETRIGGER;
-    break;
-  case COMMAND_TEMPO:
-    /* Values >0xf0 have special meanings in OctaMED */
-    if (command[1] > 0xf0) {
-      switch (command[1]) {
-      case 0xf1:
-	/* Makes a single note play twice */
-	command[0] = COMMAND_RETRIGGER;
-	command[1] = 3;
-	break;
-      case 0xf2:
-	/* Delays the start of a note by half a line */
-	command[0] = COMMAND_DELAY;
-	command[1] = 3;
-	break;
-      case 0xf3:
-	/* Works like 0FF1 except the note is played three times */
-	command[0] = COMMAND_RETRIGGER;
-	command[1] = 2;
-	break;
-      case 0xf4:
-	/* Delays the note one-third of a line */
-	command[0] = COMMAND_DELAY;
-	command[1] = 2;
-	break;
-      case 0xf5:
-	/* Delays the note two thirds of a line */
-	command[0] = COMMAND_DELAY;
-	command[1] = 4;
-	break;
-      case 0xfe:
-	/* Stops the song playing */
-	command[0] = COMMAND_TEMPO;
-	command[1] = 0;
-	break;
-      case 0xff:
-	/* Stops the note on the current track */
-	command[0] = COMMAND_VELOCITY;
-	command[1] = 0;
-	break;
-      default:
-	command[0] = 0;
-	command[1] = 0;
-	break;
-      }
-    }
+  case 0x20:
+    /* Delay */
+    command[0] = 0x1f;
+    command[1] <<= 4;
     break;
   case 0x31:
   case 0x32:
@@ -299,27 +336,23 @@ void MMD2_commands_convert(unsigned char *command, struct MMDMIDICmd3x *mmdcmd3x
   case 0x3d:
   case 0x3e:
   case 0x3f:
-    if (mmdcmd3x->ctrlr_types[command[0] - 0x31] == MCS_TYPE_STD_MSB)
-      command[0] = mmdcmd3x->ctrlr_numbers[command[0] - 0x31] + 0x80;
-    break;
+    /* MMD3x: OctaMED doesn't support value interpolation */
+    command[1] &= 0x7f;
   default:
     break;
   }
 }
 
 /* Converts a song to an MMD2 module */
-struct MMD2 *song_convert(struct song *song)
+struct MMD2 *song_convert_MMD2(struct song *song)
 {
+  nullcheck_pointer(song, song_convert_MMD2);
+
   struct MMD2 *mmd;
   int i, j, k, l;
   unsigned char commandconversion[256];
   unsigned int controllers_used = 0;
 
-  if (song == NULL) {
-    fprintf(stderr, "song_convert() called with null song\n");
-    return NULL;
-  }
-
   /* Create a table for converting song commands; values need special handling */
   for (i = 0; i < 256; i++)
     commandconversion[i] = i;
@@ -475,11 +508,11 @@ struct MMD2 *song_convert(struct song *song)
     gchar *name = g_convert(song->instruments[i]->name, strlen(song->instruments[i]->name), "ISO-8859-1", "UTF-8", NULL, NULL, NULL);
     strncpy(mmd->expdata->iinfo[i].name, name, 40);
     mmd->expdata->iinfo[i].name[39] = 0;
-    mmd->expdata->exp_smp[i].hold = song->instruments[i]->hold;
-    mmd->song->sample[i].midich = song->instruments[i]->midichannel + 1;
-    mmd->song->sample[i].midipreset = song->instruments[i]->midipreset;
-    mmd->song->sample[i].strans = song->instruments[i]->transpose;
-    mmd->song->sample[i].svol = (song->instruments[i]->defaultvelocity + 1) / 2;
+    mmd->expdata->exp_smp[i].hold = song->instruments[i]->outputs[0]->hold;
+    mmd->song->sample[i].midich = song->instruments[i]->outputs[0]->midichannel + 1;
+    mmd->song->sample[i].midipreset = song->instruments[i]->outputs[0]->midipreset;
+    mmd->song->sample[i].strans = song->instruments[i]->outputs[0]->transpose;
+    mmd->song->sample[i].svol = (song->instruments[i]->outputs[0]->defaultvelocity + 1) / 2;
     g_free(name);
   }
 
@@ -488,7 +521,7 @@ struct MMD2 *song_convert(struct song *song)
   mmd->song->trackvols = (unsigned char *)calloc(mmd->song->numtracks, sizeof(unsigned char));
   mmd->song->trackpans = (unsigned char *)calloc(mmd->song->numtracks, sizeof(unsigned char));
   for (i = 0; i < mmd->song->numtracks; i++)
-    mmd->song->trackvols[i] = (song->trackvolumes[i] + 1) / 2;
+    mmd->song->trackvols[i] = (song->tracks[i]->volume + 1) / 2;
   mmd->song->mastervol = (song->mastervolume + 1) / 2;
 
   /* SysEX dumps */
@@ -514,37 +547,49 @@ struct MMD2 *song_convert(struct song *song)
   return mmd;
 }
 
-/* Converts song command values at given location */
-void song_commands_convert(unsigned char *command)
+/* Converts a song to a standard MIDI file */
+struct smf *song_convert_smf(struct song *song)
 {
-  switch (command[0]) {
-  case 0x0c:
-    /* Fix volume range to 0..64 */
-    command[1] = (command[1] + 1) / 2;
-    break;
-  case 0x20:
-    /* Delay */
-    command[0] = 0x1f;
-    command[1] <<= 4;
-    break;
-  case 0x31:
-  case 0x32:
-  case 0x33:
-  case 0x34:
-  case 0x35:
-  case 0x36:
-  case 0x37:
-  case 0x38:
-  case 0x39:
-  case 0x3a:
-  case 0x3b:
-  case 0x3c:
-  case 0x3d:
-  case 0x3e:
-  case 0x3f:
-    /* MMD3x: OctaMED doesn't support value interpolation */
-    command[1] &= 0x7f;
-  default:
-    break;
+  nullcheck_pointer(song, song_convert_smf);
+
+  struct smf *smf;
+  struct midi *midi;
+  int i;
+  unsigned char endMTrk[3];
+  
+  smf = calloc(1, sizeof(struct smf));
+
+  /* Allocate the MThd chunk */
+  smf->MThd = calloc(1, sizeof(struct smf_MThd));
+  smf->MThd->ID[0] = 'M';
+  smf->MThd->ID[1] = 'T';
+  smf->MThd->ID[2] = 'h';
+  smf->MThd->ID[3] = 'd';
+  smf->MThd->Length = 6;
+  smf->MThd->Format = 0;
+  smf->MThd->NumTracks = 1;
+  smf->MThd->Division = 24;
+  
+  /* Allocate the MTrk chunks */
+  smf->MTrks = calloc(smf->MThd->NumTracks, sizeof(struct smf_MTrk *));
+  for (i = 0; i < smf->MThd->NumTracks; i++) {
+    smf->MTrks[i] = calloc(1, sizeof(struct smf_MTrk));
+    smf->MTrks[i]->ID[0] = 'M';
+    smf->MTrks[i]->ID[1] = 'T';
+    smf->MTrks[i]->ID[2] = 'r';
+    smf->MTrks[i]->ID[3] = 'k';
   }
+  
+  midi = midi_alloc(MIDI_BUFFER, NULL, NULL);
+  midi_open(midi, DIRECTION_BOTH);
+  player_midi_export(song, midi);
+  endMTrk[0] = 0xff;
+  endMTrk[1] = 0x2f;
+  endMTrk[2] = 0x00;
+  midi_write_raw(midi, endMTrk, 3);
+  smf->MTrks[0]->Length = midi_get_buffer_length(midi);
+  smf->MTrks[0]->Data = midi_get_buffer(midi);
+  midi_free(midi);
+  
+  return smf;
 }
