@@ -1,3 +1,9 @@
+/*
+ * - initissä yritetään avata sekä in/out että unavailablein/out
+ * - onnistuneet -> in/out
+ * - haluttujen ja onnistuneiden erotus -> unavailablein/out
+ * - haluttaessa avata jotain jos onnistuu -> in/out, muutoin unavailablein/out
+ */
 /* editor.c
  *
  * Copyright 2002-2005 Vesa Halttunen
@@ -28,23 +34,29 @@
 #include "player.h"
 #include "callbacks.h"
 
+static char *string_remove_string(char *from, char *toremove);
+static char *string_append_char(char *to, char append);
+static char *string_append_string(char *to, char *append);
+
 /* Initializes the editor */
 struct editor *editor_open()
 {
   struct editor *editor = (struct editor *)calloc(1, sizeof(struct editor));
-  
+
   int i;
-  
+
   /* Use GConf */
   editor->gconfclient = gconf_client_get_default();
   gconf_client_add_dir(editor->gconfclient, "/apps/tutka", GCONF_CLIENT_PRELOAD_NONE, NULL);
 
   /* Set defaults */
   editor->space = 1;
+  editor->receivemidiinterface = -1;
 
-  /* Get a pointer to the main thread */
+  /* Get a pointer to the main thread and lower priority to give time to the player */
   editor->thread = g_thread_self();
-  
+  g_thread_set_priority(editor->thread, G_THREAD_PRIORITY_LOW);
+
   /* Create a mutex for locking the song */
   editor->songmutex = g_mutex_new();
 
@@ -54,7 +66,7 @@ struct editor *editor_open()
     editor_close(editor);
     return NULL;
   }
-  
+
   /* Get MIDI interfaces */
   editor->nummidiinterfaces = midi_get_num_alsa_devices();
   editor->midiinterfaces = midi_get_alsa_devices();
@@ -67,14 +79,14 @@ struct editor *editor_open()
   editor->midiinterfaces[0] = midi_alloc(MIDI_NULL, NULL, "No output");
 
   /* Open the MIDI interfaces defined in preferences */
-  midi_open_defined(editor->midiinterfaces, editor->nummidiinterfaces, editor_midi_interface_get(editor, DIRECTION_IN), editor_midi_interface_get(editor, DIRECTION_OUT));
+  midi_open_defined(editor->midiinterfaces, editor->nummidiinterfaces, editor_midi_interface_get(editor, DIRECTION_IN, MIDIINTERFACES_AVAILABLE), editor_midi_interface_get(editor, DIRECTION_OUT, MIDIINTERFACES_AVAILABLE), editor_midi_interface_get(editor, DIRECTION_IN, MIDIINTERFACES_UNAVAILABLE), editor_midi_interface_get(editor, DIRECTION_OUT, MIDIINTERFACES_UNAVAILABLE));
 
   /* Some interfaces may have been removed so refresh preferences */
   editor_midi_interface_set_from_current(editor);
 
   /* Refresh the preferences window */
   gui_preferences_refresh(editor->gui);
-  
+
   editor->gconfnotifyid = gconf_client_notify_add(editor->gconfclient, "/apps/tutka", gconf_notify_function, editor->gui, NULL, NULL);
 
   return editor;
@@ -141,7 +153,7 @@ void editor_song_open(struct editor *editor, char *filename)
     /* Figure out default MIDI interface number for new instruments */
     int i = 0;
     if (editor->nummidiinterfaces > 1)
-    i = 1;
+      i = 1;
 
     editor->filename = NULL;
     editor->song = song_alloc();
@@ -171,7 +183,7 @@ void editor_close(struct editor *editor)
   nullcheck_void(editor, editor_close);
 
   int i;
-  
+
   /* Close the player */
   if (editor->player != NULL)
     player_close(editor->player);
@@ -222,12 +234,12 @@ void editor_close(struct editor *editor)
 void editor_song_midi_interface_numbers_set(struct editor *editor)
 {
   nullcheck_void(editor, editor_song_midi_interface_numbers_set);
-  
+
   int i, j;
-  
+
   if (editor->song == NULL)
     return;
-  
+
   for (i = 0; i < editor->song->numinstruments; i++) {
     struct instrument *instrument = editor->song->instruments[i];
     for (j = 0; j < instrument->numoutputs; j++) {
@@ -241,18 +253,18 @@ void editor_song_midi_interface_numbers_set(struct editor *editor)
 void editor_song_midi_interface_names_set(struct editor *editor)
 {
   nullcheck_void(editor, editor_song_midi_interface_names_set);
-  
+
   int i, j;
-  
+
   if (editor->song == NULL)
     return;
-  
+
   for (i = 0; i < editor->song->numinstruments; i++) {
     struct instrument *instrument = editor->song->instruments[i];
     for (j = 0; j < instrument->numoutputs; j++) {
       struct instrument_output *output = instrument->outputs[j];
       if (output->midiinterfacename != NULL)
-        free (output->midiinterfacename);
+	free(output->midiinterfacename);
       output->midiinterfacename = editor_midi_interface_name_get_by_number(editor, output->midiinterface);
     }
   }
@@ -264,17 +276,17 @@ int editor_midi_interface_get_by_name(struct editor *editor, char *name)
   nullcheck_int(editor, editor_midi_interface_get_by_name);
 
   int i;
-  
+
   /* The first interface is always the null interface */
   if (name == NULL)
     return 0;
-  
+
   for (i = 1; i < editor->nummidiinterfaces; i++) {
     struct midi *midi = editor->midiinterfaces[i];
     if (strcmp(midi->devicename, name) == 0)
       return i;
   }
-    
+
   /* If no interface is found use the null interface */
   return 0;
 }
@@ -472,7 +484,7 @@ void editor_midi_input(struct editor *editor, unsigned char *data)
   /* FIX: gui->tracker should not be used here directly (or at all) */
   Tracker *tracker = (Tracker *) glade_xml_get_widget(editor->gui->xml, "tracker");
   struct song *song = editor->song;
-  
+
   switch (data[0] & 0xf0) {
   case 0x80:
     /* Note off */
@@ -550,7 +562,7 @@ void editor_reset_time(struct editor *editor)
 void editor_song_lock(struct editor *editor)
 {
   nullcheck_void(editor, editor_song_lock);
-  
+
   g_mutex_lock(editor->songmutex);
 }
 
@@ -558,8 +570,8 @@ void editor_song_lock(struct editor *editor)
 void editor_song_unlock(struct editor *editor)
 {
   nullcheck_void(editor, editor_song_unlock)
-  
-  g_mutex_unlock(editor->songmutex);
+
+    g_mutex_unlock(editor->songmutex);
 }
 
 /* Checks if the maximum number of tracks in the song has changed */
@@ -568,11 +580,11 @@ int editor_song_maxtracks_check(struct editor *editor)
   nullcheck_int(editor, editor_song_maxtracks_check);
 
   int oldmax, v;
-  
+
   g_mutex_lock(editor->songmutex);
   oldmax = editor->song->maxtracks;
   v = song_check_maxtracks(editor->song);
-  
+
   if (v)
     player_trackstatus_create(editor->player, oldmax);
   g_mutex_unlock(editor->songmutex);
@@ -814,7 +826,7 @@ void editor_song_block_current_insert_line_track_all(struct editor *editor)
   /* Move lines downwards */
   b = block_copy(song->blocks[editor->block], 0, editor->line, song->blocks[editor->block]->tracks - 1, song->blocks[editor->block]->length - 1);
   block_paste(song->blocks[editor->block], b, 0, editor->line + 1);
-  
+
   /* Clear the inserted line */
   for (j = 0; j < song->blocks[editor->block]->tracks; j++) {
     block_set_note(song->blocks[editor->block], editor->line, j, 0, 0, 0);
@@ -860,12 +872,12 @@ void editor_song_block_current_delete_line_track_all(struct editor *editor)
 
   g_mutex_lock(editor->songmutex);
   song = editor->song;
-  
+
   /* Move lines upwards */
   b = block_copy(song->blocks[editor->block], 0, editor->line + 1, song->blocks[editor->block]->tracks - 1, song->blocks[editor->block]->length - 1);
   block_paste(song->blocks[editor->block], b, 0, editor->line);
   block_free(b);
-  
+
   /* Clear the last line */
   for (i = 0; i < song->blocks[editor->block]->tracks; i++) {
     block_set_note(song->blocks[editor->block], song->blocks[editor->block]->length - 1, i, 0, 0, 0);
@@ -938,20 +950,20 @@ unsigned int editor_song_track_solo_get(struct editor *editor, unsigned int trac
 void editor_song_track_name_set(struct editor *editor, unsigned int track, char *name)
 {
   nullcheck_void(editor, editor_song_track_name_set);
-  
+
   g_mutex_lock(editor->songmutex);
   if (name != NULL)
     editor->song->tracks[track]->name = strdup(name);
   else
     editor->song->tracks[track]->name = NULL;
-  g_mutex_unlock(editor->songmutex);  
+  g_mutex_unlock(editor->songmutex);
 }
 
 /* Returns the name of a track */
 char *editor_song_track_name_get(struct editor *editor, unsigned int track)
-{ 
+{
   nullcheck_pointer(editor, editor_song_track_name_get);
-  
+
   return editor->song->tracks[track]->name;
 }
 
@@ -959,12 +971,12 @@ char *editor_song_track_name_get(struct editor *editor, unsigned int track)
 void editor_song_block_current_copyarea(struct editor *editor, int starttrack, int startline, int endtrack, int endline)
 {
   nullcheck_void(editor, editor_song_block_current_copyarea);
-  
+
   g_mutex_lock(editor->songmutex);
   /* Free the previous copy buffer if one exists */
   if (editor->copyarea)
     block_free(editor->copyarea);
-  
+
   editor->copyarea = block_copy(editor->song->blocks[editor->block], starttrack, startline, endtrack, endline);
   g_mutex_unlock(editor->songmutex);
 }
@@ -984,7 +996,7 @@ void editor_song_block_current_copyarea_paste(struct editor *editor, int track,
 void editor_song_block_current_copytrack(struct editor *editor, int track)
 {
   nullcheck_void(editor, editor_song_block_current_copytrack);
-  
+
   g_mutex_lock(editor->songmutex);
   /* Free the previous copy buffer if one exists */
   if (editor->copytrack)
@@ -1009,7 +1021,7 @@ void editor_song_block_current_copytrack_paste(struct editor *editor, int track)
 void editor_song_block_current_copyblock(struct editor *editor)
 {
   nullcheck_void(editor, editor_song_block_current_copyblock);
-  
+
   g_mutex_lock(editor->songmutex);
   if (editor->copyblock)
     block_free(editor->copyblock);
@@ -1040,7 +1052,7 @@ unsigned int editor_song_nummessages_get(struct editor *editor)
 void editor_song_message_insert(struct editor *editor, unsigned int position)
 {
   nullcheck_void(editor, editor_song_message_insert);
-  
+
   g_mutex_lock(editor->songmutex);
   song_insert_message(editor->song, position);
   g_mutex_unlock(editor->songmutex);
@@ -1050,7 +1062,7 @@ void editor_song_message_insert(struct editor *editor, unsigned int position)
 void editor_song_message_delete(struct editor *editor, unsigned int position)
 {
   nullcheck_void(editor, editor_song_message_delete);
-  
+
   g_mutex_lock(editor->songmutex);
   song_delete_message(editor->song, position);
   g_mutex_unlock(editor->songmutex);
@@ -1060,35 +1072,53 @@ void editor_song_message_delete(struct editor *editor, unsigned int position)
 void editor_song_message_send(struct editor *editor, unsigned int position)
 {
   nullcheck_void(editor, editor_song_message_send);
-  
+
   int i;
-  
+
   /* Should midi_write_raw be used here directly? */
   g_mutex_lock(editor->songmutex);
   for (i = 1; i < editor->nummidiinterfaces; i++)
-    midi_write_raw(editor->midiinterfaces[i], editor->song->messages[position]->data, editor->song->messages[position]->length);  
+    midi_write_raw(editor->midiinterfaces[i], editor->song->messages[position]->data, editor->song->messages[position]->length);
   g_mutex_unlock(editor->songmutex);
 }
 
 /* Receives a message to the given position */
-void editor_song_message_receive(struct editor *editor, unsigned int position, unsigned char autostop)
+unsigned int editor_song_message_receive(struct editor *editor)
 {
-  nullcheck_void(editor, editor_song_message_receive);
+  nullcheck_int(editor, editor_song_message_receive);
 
-  int i;
-  
-  /* Should midi_read be used here directly? */
-  g_mutex_lock(editor->songmutex);
-  for (i = 1; i < editor->nummidiinterfaces; i++)
-    midi_read_system_exclusive(editor->midiinterfaces[i], editor->song->messages[position], autostop);
-  g_mutex_unlock(editor->songmutex);
+  if (editor->receiveactive) {
+    struct message *message = editor->song->messages[editor->receivemessage];
+
+    /* Read as many SysEx bytes as available */
+    int i = midi_read_system_exclusive(editor->midiinterfaces[editor->receivemidiinterface], message, editor->receivedbytes, editor->receiveautostop);
+
+    /* If the amount of bytes is negative last bytes have been received */
+    if (i > 0)
+      editor->receivedbytes += i;
+    else if (i < 0) {
+      /* -i is the actual number of bytes received */
+      editor->receivedbytes -= i;
+
+      on_button_receive_stop_clicked(NULL, editor->gui);
+      return 1;
+    } else {
+      /* No bytes received: if autostop is not used and as many bytes received as requested, quit */
+      if (!editor->receiveautostop && editor->receivedbytes == message->length) {
+	on_button_receive_stop_clicked(NULL, editor->gui);
+	return 1;
+      }
+    }
+  }
+
+  return 0;
 }
 
 /* Deletes the current section */
 void editor_song_section_current_delete(struct editor *editor)
 {
   nullcheck_void(editor, editor_song_section_current_delete);
-  
+
   g_mutex_lock(editor->songmutex);
   song_delete_section(editor->song, editor->section);
 
@@ -1120,7 +1150,7 @@ void editor_song_section_insert(struct editor *editor, unsigned int position)
 unsigned int editor_song_section_get(struct editor *editor, unsigned int position)
 {
   nullcheck_int(editor, editor_song_section_get);
-  
+
   return editor->song->sections[position];
 }
 
@@ -1128,7 +1158,7 @@ unsigned int editor_song_section_get(struct editor *editor, unsigned int positio
 void editor_song_playseq_current_delete_current(struct editor *editor)
 {
   nullcheck_void(editor, editor_song_playseq_current_delete_current);
-  
+
   g_mutex_lock(editor->songmutex);
   playseq_delete(editor->song->playseqs[editor->playseq], editor->position);
   editor_refresh_playseq_and_block(editor);
@@ -1143,9 +1173,9 @@ void editor_song_playseq_current_delete_current(struct editor *editor)
 void editor_song_playseq_current_insert(struct editor *editor, unsigned int position)
 {
   nullcheck_void(editor, editor_song_playseq_current_insert);
-  
+
   g_mutex_lock(editor->songmutex);
-  playseq_insert(editor->song->playseqs[editor->playseq], position);  
+  playseq_insert(editor->song->playseqs[editor->playseq], position);
   g_mutex_unlock(editor->songmutex);
 }
 
@@ -1153,7 +1183,7 @@ void editor_song_playseq_current_insert(struct editor *editor, unsigned int posi
 void editor_song_playseq_insert(struct editor *editor, unsigned int position)
 {
   nullcheck_void(editor, editor_song_playseq_insert);
-  
+
   g_mutex_lock(editor->songmutex);
   song_insert_playseq(editor->song, position);
   g_mutex_unlock(editor->songmutex);
@@ -1174,7 +1204,7 @@ void editor_song_playseq_current_delete(struct editor *editor)
 
   g_mutex_lock(editor->songmutex);
   song_delete_playseq(editor->song, editor->playseq);
-  editor_refresh_playseq_and_block(editor);  
+  editor_refresh_playseq_and_block(editor);
   g_mutex_unlock(editor->songmutex);
 }
 
@@ -1182,7 +1212,7 @@ void editor_song_playseq_current_delete(struct editor *editor)
 unsigned int editor_song_playseq_current_block_get(struct editor *editor, unsigned int position)
 {
   nullcheck_int(editor, editor_song_playseq_current_block_get);
-  
+
   return editor->song->playseqs[editor->playseq]->blocknumbers[position];
 }
 
@@ -1190,7 +1220,7 @@ unsigned int editor_song_playseq_current_block_get(struct editor *editor, unsign
 void editor_song_block_insert(struct editor *editor, unsigned int position, unsigned int block)
 {
   nullcheck_void(editor, editor_song_block_insert);
-  
+
   g_mutex_lock(editor->songmutex);
   song_insert_block(editor->song, position, block);
   g_mutex_unlock(editor->songmutex);
@@ -1228,7 +1258,7 @@ void editor_song_instrument_current_name_set(struct editor *editor, char *name)
   if (editor->song->instruments[editor->instrument]->name != NULL)
     free(editor->song->instruments[editor->instrument]->name);
 
-  editor->song->instruments[editor->instrument]->name = strdup(name);  
+  editor->song->instruments[editor->instrument]->name = strdup(name);
   g_mutex_unlock(editor->songmutex);
 }
 
@@ -1237,14 +1267,14 @@ char *editor_song_instrument_current_name_get(struct editor *editor)
 {
   nullcheck_pointer(editor, editor_song_instrument_current_name_get);
 
-  return editor->song->instruments[editor->instrument]->name; 
+  return editor->song->instruments[editor->instrument]->name;
 }
 
 /* Saves the current song */
 void editor_song_save(struct editor *editor, char *filename)
 {
   nullcheck_void(editor, editor_song_save);
-  
+
   gchar *path;
   int l;
 
@@ -1288,7 +1318,7 @@ void editor_song_save(struct editor *editor, char *filename)
 void editor_song_name_set(struct editor *editor, char *name)
 {
   nullcheck_void(editor, editor_song_name_set);
-  
+
   g_mutex_lock(editor->songmutex);
   /* Free the old name if one exists */
   if (editor->song->name != NULL)
@@ -1322,7 +1352,7 @@ void editor_song_block_current_transpose(struct editor *editor, int instrument,
 void editor_song_expandshrink(struct editor *editor, int factor)
 {
   nullcheck_void(editor, editor_block_song_expandshrink);
-  
+
   g_mutex_lock(editor->songmutex);
   song_expandshrink(editor->song, factor);
   g_mutex_unlock(editor->songmutex);
@@ -1332,7 +1362,7 @@ void editor_song_expandshrink(struct editor *editor, int factor)
 void editor_song_block_current_expandshrink(struct editor *editor, int factor, int starttrack, int startline, int endtrack, int endline)
 {
   nullcheck_void(editor, editor_block_current_expandshrink);
-  
+
   g_mutex_lock(editor->songmutex);
   block_expandshrink(editor->song->blocks[editor->block], factor, starttrack, startline, endtrack, endline);
   g_mutex_unlock(editor->songmutex);
@@ -1352,7 +1382,7 @@ void editor_song_changeinstrument(struct editor *editor, int from, int to, int s
 void editor_song_block_current_changeinstrument(struct editor *editor, int from, int to, int swap, int starttrack, int startline, int endtrack, int endline)
 {
   nullcheck_void(editor, editor_block_current_changeinstrument);
-  
+
   g_mutex_lock(editor->songmutex);
   block_changeinstrument(editor->song->blocks[editor->block], from, to, swap, starttrack, startline, endtrack, endline);
   g_mutex_unlock(editor->songmutex);
@@ -1378,7 +1408,7 @@ void editor_song_sendsync_set(struct editor *editor, unsigned int sendsync)
   nullcheck_void(editor, editor_song_sendsync_set);
 
   g_mutex_lock(editor->songmutex);
-  editor->song->sendsync = sendsync;  
+  editor->song->sendsync = sendsync;
   g_mutex_unlock(editor->songmutex);
 }
 
@@ -1395,7 +1425,7 @@ void editor_song_message_autosend_set(struct editor *editor, unsigned int messag
 /* Saves a MIDI message */
 void editor_song_message_save(struct editor *editor, unsigned int message, char *filename)
 {
-  nullcheck_void(editor, editor_song_message_save); 
+  nullcheck_void(editor, editor_song_message_save);
   gchar *path;
 
   /* Save the message */
@@ -1437,7 +1467,7 @@ void editor_song_mute(struct editor *editor)
 {
   nullcheck_void(editor, editor_song_mute);
   int i, unmute = 1;
-  
+
   /* Check if all tracks are already muted */
   g_mutex_lock(editor->songmutex);
   for (i = 0; i < editor->song->maxtracks; i++)
@@ -1509,7 +1539,7 @@ void editor_song_playseq_current_set(struct editor *editor, unsigned int block)
 void editor_song_block_current_name_set(struct editor *editor, char *name)
 {
   nullcheck_void(editor, editor_song_block_current_name_set);
-  
+
   g_mutex_lock(editor->songmutex);
   block_set_name(editor->song->blocks[editor->block], name);
   g_mutex_unlock(editor->songmutex);
@@ -1519,7 +1549,7 @@ void editor_song_block_current_name_set(struct editor *editor, char *name)
 char *editor_song_block_current_name_get(struct editor *editor)
 {
   nullcheck_pointer(editor, editor_song_block_current_name_get);
-  
+
   return editor->song->blocks[editor->block]->name;
 }
 
@@ -1527,7 +1557,7 @@ char *editor_song_block_current_name_get(struct editor *editor)
 char *editor_song_block_name_get(struct editor *editor, unsigned int block)
 {
   nullcheck_pointer(editor, editor_song_block_name_get);
-  
+
   return editor->song->blocks[block]->name;
 }
 
@@ -1535,7 +1565,7 @@ char *editor_song_block_name_get(struct editor *editor, unsigned int block)
 void editor_song_block_current_tracks_set(struct editor *editor, unsigned int tracks)
 {
   nullcheck_void(editor, editor_song_block_current_tracks_set);
-  
+
   g_mutex_lock(editor->songmutex);
   block_set_tracks(editor->song->blocks[editor->block], tracks);
   g_mutex_unlock(editor->songmutex);
@@ -1545,7 +1575,7 @@ void editor_song_block_current_tracks_set(struct editor *editor, unsigned int tr
 unsigned int editor_song_block_tracks_get(struct editor *editor, unsigned int block)
 {
   nullcheck_int(editor, editor_song_block_tracks_get);
-  
+
   return editor->song->blocks[block]->tracks;
 }
 
@@ -1553,9 +1583,9 @@ unsigned int editor_song_block_tracks_get(struct editor *editor, unsigned int bl
 void editor_song_block_current_length_set(struct editor *editor, unsigned int length)
 {
   int i = editor->line;
-  
+
   nullcheck_void(editor, editor_song_block_current_length_set);
-  
+
   g_mutex_lock(editor->songmutex);
   block_set_length(editor->song->blocks[editor->block], length);
   editor->line = -1;
@@ -1567,7 +1597,7 @@ void editor_song_block_current_length_set(struct editor *editor, unsigned int le
 unsigned int editor_song_block_length_get(struct editor *editor, unsigned int block)
 {
   nullcheck_int(editor, editor_song_block_length_get);
-  
+
   return editor->song->blocks[block]->length;
 }
 
@@ -1575,9 +1605,9 @@ unsigned int editor_song_block_length_get(struct editor *editor, unsigned int bl
 void editor_song_block_current_commandpages_set(struct editor *editor, unsigned int commandpages)
 {
   int i = editor->cmdpage;
-  
+
   nullcheck_void(editor, editor_song_block_current_commandpages_set);
-  
+
   g_mutex_lock(editor->songmutex);
   block_set_commandpages(editor->song->blocks[editor->block], commandpages);
   editor->cmdpage = -1;
@@ -1589,7 +1619,7 @@ void editor_song_block_current_commandpages_set(struct editor *editor, unsigned
 unsigned int editor_song_block_commandpages_get(struct editor *editor, unsigned int block)
 {
   nullcheck_int(editor, editor_song_block_commandpages_get);
-  
+
   return editor->song->blocks[block]->commandpages;
 }
 
@@ -1610,8 +1640,8 @@ void editor_song_message_name_set(struct editor *editor, unsigned int message, c
 char *editor_song_message_name_get(struct editor *editor, unsigned int message)
 {
   nullcheck_pointer(editor, editor_song_message_name_get);
-  
-  return editor->song->messages[message]->name; 
+
+  return editor->song->messages[message]->name;
 }
 
 /* Sets the length of a MIDI message */
@@ -1646,7 +1676,7 @@ void editor_song_print(struct editor *editor)
   nullcheck_void(editor, editor_song_print);
 
   g_mutex_lock(editor->songmutex);
-  song_print(editor->song, NULL);  
+  song_print(editor->song, NULL);
   g_mutex_unlock(editor->songmutex);
 }
 
@@ -1654,7 +1684,7 @@ void editor_song_print(struct editor *editor)
 void editor_song_tempo_set(struct editor *editor, unsigned int tempo)
 {
   nullcheck_void(editor, editor_song_tempo_set);
- 
+
   g_mutex_lock(editor->songmutex);
   editor->song->tempo = tempo;
   g_mutex_unlock(editor->songmutex);
@@ -1664,7 +1694,7 @@ void editor_song_tempo_set(struct editor *editor, unsigned int tempo)
 unsigned int editor_song_tempo_get(struct editor *editor)
 {
   nullcheck_int(editor, editor_song_tempo_get);
- 
+
   return editor->song->tempo;
 }
 
@@ -1672,7 +1702,7 @@ unsigned int editor_song_tempo_get(struct editor *editor)
 void editor_song_ticksperline_set(struct editor *editor, unsigned int ticksperline)
 {
   nullcheck_void(editor, editor_song_ticksperline_set);
- 
+
   g_mutex_lock(editor->songmutex);
   editor->song->ticksperline = ticksperline;
   g_mutex_unlock(editor->songmutex);
@@ -1682,7 +1712,7 @@ void editor_song_ticksperline_set(struct editor *editor, unsigned int ticksperli
 unsigned int editor_song_ticksperline_get(struct editor *editor)
 {
   nullcheck_int(editor, editor_song_ticksperline_get);
- 
+
   return editor->song->ticksperline;
 }
 
@@ -1690,7 +1720,7 @@ unsigned int editor_song_ticksperline_get(struct editor *editor)
 struct block *editor_song_block_current_get(struct editor *editor)
 {
   nullcheck_pointer(editor, editor_song_block_current_get);
-  
+
   return editor->song->blocks[editor->block];
 }
 
@@ -1698,7 +1728,7 @@ struct block *editor_song_block_current_get(struct editor *editor)
 char *editor_song_name_get(struct editor *editor)
 {
   nullcheck_pointer(editor, editor_song_name_get);
-  
+
   return editor->song->name;
 }
 
@@ -1706,7 +1736,7 @@ char *editor_song_name_get(struct editor *editor)
 unsigned int editor_song_sendsync_get(struct editor *editor)
 {
   nullcheck_int(editor, editor_song_sendsync_get);
-  
+
   return editor->song->sendsync;
 }
 
@@ -1714,7 +1744,7 @@ unsigned int editor_song_sendsync_get(struct editor *editor)
 void editor_scheduler_set(struct editor *editor, unsigned int mode)
 {
   nullcheck_void(editor, editor_scheduler_set);
-  
+
   gconf_client_set_int(editor->gconfclient, "/apps/tutka/scheduler", mode, NULL);
 }
 
@@ -1724,7 +1754,7 @@ unsigned int editor_scheduler_get(struct editor *editor)
   nullcheck_int(editor, editor_scheduler_get);
 
   int scheduler = gconf_client_get_int(editor->gconfclient, "/apps/tutka/scheduler", NULL);
-  
+
   /* For backwards compatibility: don't allow the scheduler to be 0 */
   if (scheduler == SCHED_NONE) {
     scheduler = SCHED_RTC;
@@ -1735,25 +1765,39 @@ unsigned int editor_scheduler_get(struct editor *editor)
 }
 
 /* Sets the names of the MIDI interfaces to enable separated by ; */
-void editor_midi_interface_set(struct editor *editor, char *midi_interface, unsigned int direction)
+void editor_midi_interface_set(struct editor *editor, char *midi_interface, unsigned int direction, unsigned int mode)
 {
   nullcheck_void(editor, editor_midi_interface_set);
 
-  if (direction == DIRECTION_IN)
-    gconf_client_set_string(editor->gconfclient, "/apps/tutka/midiinterfaces/in", midi_interface, NULL);
-  else
-    gconf_client_set_string(editor->gconfclient, "/apps/tutka/midiinterfaces/out", midi_interface, NULL);    
+  if (mode == MIDIINTERFACES_AVAILABLE) {
+    if (direction == DIRECTION_IN)
+      gconf_client_set_string(editor->gconfclient, "/apps/tutka/midiinterfaces/in", midi_interface, NULL);
+    else
+      gconf_client_set_string(editor->gconfclient, "/apps/tutka/midiinterfaces/out", midi_interface, NULL);
+  } else {
+    if (direction == DIRECTION_IN)
+      gconf_client_set_string(editor->gconfclient, "/apps/tutka/midiinterfaces/unavailablein", midi_interface, NULL);
+    else
+      gconf_client_set_string(editor->gconfclient, "/apps/tutka/midiinterfaces/unavailableout", midi_interface, NULL);
+  }
 }
 
-/* Returns the names of the MIDI interface to enable separated by ; */
-char *editor_midi_interface_get(struct editor *editor, unsigned int direction)
+/* Returns the names of the MIDI interfaces to enable separated by ; */
+char *editor_midi_interface_get(struct editor *editor, unsigned int direction, unsigned int mode)
 {
   nullcheck_pointer(editor, editor_midi_interface_get);
-  
-  if (direction == DIRECTION_IN)
-    return gconf_client_get_string(editor->gconfclient, "/apps/tutka/midiinterfaces/in", NULL);
-  else
-    return gconf_client_get_string(editor->gconfclient, "/apps/tutka/midiinterfaces/out", NULL);
+
+  if (mode == MIDIINTERFACES_AVAILABLE) {
+    if (direction == DIRECTION_IN)
+      return gconf_client_get_string(editor->gconfclient, "/apps/tutka/midiinterfaces/in", NULL);
+    else
+      return gconf_client_get_string(editor->gconfclient, "/apps/tutka/midiinterfaces/out", NULL);
+  } else {
+    if (direction == DIRECTION_IN)
+      return gconf_client_get_string(editor->gconfclient, "/apps/tutka/midiinterfaces/unavailablein", NULL);
+    else
+      return gconf_client_get_string(editor->gconfclient, "/apps/tutka/midiinterfaces/unavailableout", NULL);
+  }
 }
 
 /* Marks the enabled interfaces as enabled in the preferences */
@@ -1761,57 +1805,177 @@ void editor_midi_interface_set_from_current(struct editor *editor)
 {
   nullcheck_void(editor, editor_midi_interface_preferences_refresh);
 
-  int i, inlen = 0, outlen = 0;
-  char *instring = NULL, *outstring = NULL;
-  
-  for (i = 1; i < editor->nummidiinterfaces; i++) {
-    if (editor->midiinterfaces[i]->direction & DIRECTION_IN) {
-      /* Length of device name + 1 (for the ; separator) */
-      int l = strlen(editor->midiinterfaces[i]->devicename) + 1;
+  int i, j, l, last;
+  char *instring = editor_midi_interface_get(editor, DIRECTION_IN, MIDIINTERFACES_AVAILABLE);
+  char *outstring = editor_midi_interface_get(editor, DIRECTION_OUT, MIDIINTERFACES_AVAILABLE);
+  char *unavailableinstring = editor_midi_interface_get(editor, DIRECTION_IN, MIDIINTERFACES_UNAVAILABLE);
+  char *unavailableoutstring = editor_midi_interface_get(editor, DIRECTION_OUT, MIDIINTERFACES_UNAVAILABLE);
+  char *current = NULL, *next = NULL;
+  unsigned int direction, unavailabledirection;
 
-      /* Reallocate */
-      instring = realloc(instring, inlen + l);
-      strcpy(instring + inlen, editor->midiinterfaces[i]->devicename);
-      inlen += l;
-      instring[inlen - 1] = ';';
+  /* Add a ; after unavailable strings if they exist */
+  if (unavailableinstring != NULL && strlen(unavailableinstring) > 0)
+    unavailableinstring = string_append_char(unavailableinstring, ';');
+
+  if (unavailableoutstring != NULL && strlen(unavailableoutstring) > 0)
+    unavailableoutstring = string_append_char(unavailableoutstring, ';');
+
+  /* Move all the interfaces that do not exist to unavailable */
+  for (j = 0; j < 2; j++) {
+    last = 0;
+
+    switch (j) {
+    case 0:
+      current = instring;
+      next = instring;
+      direction = DIRECTION_IN;
+      break;
+    case 1:
+      current = outstring;
+      next = outstring;
+      direction = DIRECTION_OUT;
+      break;
     }
 
-    if (editor->midiinterfaces[i]->direction & DIRECTION_OUT) {
-      /* Length of device name + 1 (for the ; separator) */
-      int l = strlen(editor->midiinterfaces[i]->devicename) + 1;
+    /* If there are no devices to go through do nothing */
+    if (strlen(current) == 0)
+      last = 1;
+    
+    while (!last) {
+      /* Find the end of the device name */
+      while (*next != ';' && *next != 0)
+	next++;
+      if (*next == 0)
+        last = 1;
+      else
+	*next = 0;
+      
+      /* Find the device from the MIDI interfaces list */
+      for (i = 1; i < editor->nummidiinterfaces; i++) {
+	if (editor->midiinterfaces[i]->devicename != NULL && strcmp(editor->midiinterfaces[i]->devicename, current) == 0)
+	  break;
+      }
 
-      /* Reallocate */
-      outstring = realloc(outstring, outlen + l);
-      strcpy(outstring + outlen, editor->midiinterfaces[i]->devicename);
-      outlen += l;
-      outstring[outlen - 1] = ';';
+      /* If the interface didn't exist move it to unavailable */
+      if (i >= editor->nummidiinterfaces) {
+        if (j == 0) {
+          unavailableinstring = string_append_string(unavailableinstring, current);
+          unavailableinstring = string_append_char(unavailableinstring, ';');
+        } else {
+          unavailableoutstring = string_append_string(unavailableoutstring, current);
+          unavailableoutstring = string_append_char(unavailableoutstring, ';');
+        }
+      }
+
+      current = ++next;
     }
   }
 
-  if (instring == NULL)
-    /* If no interface is active use "" */
-    instring = (char *)calloc(1, sizeof(char));
-  else {
-    /* Otherwise remove last separator */
-    instring = realloc(instring, inlen);
-    instring[inlen - 1] = 0;
-  }
-
-  if (outstring == NULL)
-    /* If no interface is active use "" */
-    outstring = (char *)calloc(1, sizeof(char));
-  else {
-    /* Otherwise remove last separator */
-    outstring = realloc(outstring, outlen);
-    outstring[outlen - 1] = 0;
-  }
-
-  /* Set the midi interfaces */
-  editor_midi_interface_set(editor, instring, DIRECTION_IN);
-  editor_midi_interface_set(editor, outstring, DIRECTION_OUT);
-  
+  /* Free the old in/out strings */
   free(instring);
   free(outstring);
+  instring = NULL;
+  outstring = NULL;
+  
+  /* Check all MIDI interfaces for availability */
+  for (i = 1; i < editor->nummidiinterfaces; i++) {
+    direction = editor->midiinterfaces[i]->direction;
+    unavailabledirection = editor->midiinterfaces[i]->unavailabledirection;
+
+    if (direction & DIRECTION_IN) {
+      instring = string_append_string(instring, editor->midiinterfaces[i]->devicename);
+      instring = string_append_char(instring, ';');
+    }
+
+    if (direction & DIRECTION_OUT) {
+      outstring = string_append_string(outstring, editor->midiinterfaces[i]->devicename);
+      outstring = string_append_char(outstring, ';');
+    }
+
+    if (unavailabledirection & DIRECTION_IN) {
+      unavailableinstring = string_append_string(unavailableinstring, editor->midiinterfaces[i]->devicename);
+      unavailableinstring = string_append_char(unavailableinstring, ';');
+    }
+
+    if (unavailabledirection & DIRECTION_OUT) {
+      unavailableoutstring = string_append_string(unavailableoutstring, editor->midiinterfaces[i]->devicename);
+      unavailableoutstring = string_append_char(unavailableoutstring, ';');
+    }
+
+    editor->midiinterfaces[i]->unavailabledirection = 0;
+  }
+
+  /* If the unavailable string is empty use ; - otherwise it always ends with ; at this point */
+  if (unavailableinstring == NULL)
+    unavailableinstring = (char *)calloc(0, sizeof(char));
+  if (unavailableoutstring == NULL)
+    unavailableoutstring = (char *)calloc(0, sizeof(char));
+
+  if (strlen(unavailableinstring) == 0)
+    unavailableinstring = string_append_char(unavailableinstring, ';');
+  if (strlen(unavailableoutstring) == 0)
+    unavailableoutstring = string_append_char(unavailableoutstring, ';');
+
+  /* Remove all active interfaces from unavailable lists */
+  for (i = 1; i < editor->nummidiinterfaces; i++) {
+    /* Append ; to the end of the device name for searching */
+    char *devicename = string_append_char(strdup(editor->midiinterfaces[i]->devicename), ';');
+
+    direction = editor->midiinterfaces[i]->direction;
+
+    /* If input is active it must not be found from unavailable input devices */
+    if (direction & DIRECTION_IN)
+      unavailableinstring = string_remove_string(unavailableinstring, devicename);
+
+    /* If output is active it must not be found from unavailable output devices */
+    if (direction & DIRECTION_OUT)
+      unavailableoutstring = string_remove_string(unavailableoutstring, devicename);
+  }
+
+  /* If no interfaces use "", otherwise remove last separator */
+  if (instring == NULL)
+    instring = (char *)calloc(1, sizeof(char));
+  else {
+    l = strlen(instring);
+    if (l > 0) {
+      instring = realloc(instring, l);
+      instring[l - 1] = 0;
+    }
+  }
+    
+  if (outstring == NULL)
+    outstring = (char *)calloc(1, sizeof(char));
+  else {
+    l = strlen(outstring);
+    if (l > 0) {
+      outstring = realloc(outstring, l);
+      outstring[l - 1] = 0;
+    }
+  }
+
+  /* Remove last separator from unavailable strings if there is any */
+  l = strlen(unavailableinstring);
+  if (l > 0) {
+    unavailableinstring = realloc(unavailableinstring, l);
+    unavailableinstring[l - 1] = 0;
+  }
+
+  l = strlen(unavailableoutstring);
+  if (l > 0) {
+    unavailableoutstring = realloc(unavailableoutstring, l);
+    unavailableoutstring[l - 1] = 0;
+  }
+
+  /* Set the MIDI interfaces */
+  editor_midi_interface_set(editor, instring, DIRECTION_IN, MIDIINTERFACES_AVAILABLE);
+  editor_midi_interface_set(editor, outstring, DIRECTION_OUT, MIDIINTERFACES_AVAILABLE);
+  editor_midi_interface_set(editor, unavailableinstring, DIRECTION_IN, MIDIINTERFACES_UNAVAILABLE);
+  editor_midi_interface_set(editor, unavailableoutstring, DIRECTION_OUT, MIDIINTERFACES_UNAVAILABLE);
+
+  free(instring);
+  free(outstring);
+  free(unavailableinstring);
+  free(unavailableoutstring);
 }
 
 /* Enables or disables a MIDI interface by name */
@@ -1819,19 +1983,18 @@ unsigned int editor_midi_interface_status_set(struct editor *editor, char *devic
 {
   nullcheck_int(editor, editor_midi_interface_status_set);
   nullcheck_int(devicename, editor_midi_interface_status_set);
-  
   int i, status = 0;
-  
+
   /* Search for the specified device, open or close as requested */
   for (i = 1; i < editor->nummidiinterfaces; i++)
     if (strcmp(editor->midiinterfaces[i]->devicename, devicename) == 0) {
       if (enabled)
-        status = midi_open(editor->midiinterfaces[i], direction);
+	status = midi_open(editor->midiinterfaces[i], direction);
       else
-        midi_close(editor->midiinterfaces[i], direction);
+	midi_close(editor->midiinterfaces[i], direction);
       break;
     }
-  
+
   editor_midi_interface_set_from_current(editor);
   return status;
 }
@@ -1840,7 +2003,7 @@ unsigned int editor_midi_interface_status_set(struct editor *editor, char *devic
 char *editor_song_path_get(struct editor *editor)
 {
   nullcheck_pointer(editor, editor_song_path_get);
-  
+
   return gconf_client_get_string(editor->gconfclient, "/apps/tutka/songpath", NULL);
 }
 
@@ -1848,7 +2011,7 @@ char *editor_song_path_get(struct editor *editor)
 char *editor_message_path_get(struct editor *editor)
 {
   nullcheck_pointer(editor, editor_message_path_get);
-  
+
   return gconf_client_get_string(editor->gconfclient, "/apps/tutka/messagepath", NULL);
 }
 
@@ -1872,7 +2035,7 @@ void editor_player_stop(struct editor *editor)
 void editor_player_all_notes_stop(struct editor *editor)
 {
   nullcheck_void(editor, editor_player_all_notes_stop);
-  
+
   player_stop_all_notes(editor->player);
 }
 
@@ -1880,7 +2043,7 @@ void editor_player_all_notes_stop(struct editor *editor)
 void editor_player_pitch_reset(struct editor *editor)
 {
   nullcheck_void(editor, editor_player_pitch_reset);
-  
+
   player_reset_pitch(editor->player);
 }
 
@@ -1896,7 +2059,7 @@ void editor_player_trackstatus_create(struct editor *editor, unsigned int tracks
 unsigned int editor_player_mode_get(struct editor *editor)
 {
   nullcheck_int(editor, editor_player_mode_get);
-  
+
   return editor->player->mode;
 }
 
@@ -1907,3 +2070,57 @@ void editor_player_play_note(struct editor *editor, unsigned char note, unsigned
 
   player_play_note(editor->player, editor->instrument, editor->octave * 12 + note, 127, track);
 }
+
+/* Helper functions */
+static char *string_remove_string(char *from, char *toremove)
+{
+  int fl = strlen(from) + 1;
+  int l = strlen(toremove);
+  int ml;
+
+  char *match = strstr(from, toremove);
+  while (match != NULL) {
+    ml = strlen(match);
+    memmove(match, match + l, ml - l);
+    fl -= l;
+    
+    /* Make sure the string won't completely disappear */
+    if (fl < 1)
+      fl = 1;
+    
+    from = realloc(from, fl);
+    from[fl - 1] = 0;
+    
+    match = strstr(from, toremove);
+  }
+
+  return from;
+}
+
+static char *string_append_char(char *to, char append)
+{
+  int l = 0;
+
+  if (to != NULL)
+    l = strlen(to);
+
+  to = realloc(to, l + 2);
+  to[l++] = append;
+  to[l] = 0;
+  
+  return to;
+}
+
+static char *string_append_string(char *to, char *append)
+{
+  int l1 = 0;
+  int l2 = strlen(append);
+  
+  if (to != NULL)
+    l1 = strlen(to);
+
+  to = realloc(to, l1 + l2 + 1);
+  strcpy(to + l1, append);
+
+  return to;
+}
