@@ -245,7 +245,7 @@ struct gui *gui_open(struct editor *editor)
   gtk_combo_box_set_active(GTK_COMBO_BOX(glade_xml_get_widget(gui->xml, "combobox_changeinstrument_area")), 0);
   gtk_combo_box_set_active(GTK_COMBO_BOX(glade_xml_get_widget(gui->xml, "combobox_expandshrink_area")), 0);
 
-  /* Create the MIDI interface selector (libglade doesn't do this if no items are defined) */
+  /* Create the MIDI interface selectors (libglade doesn't do this if no items are defined) */
   w = glade_xml_get_widget(gui->xml, "combobox_instrumentproperties_midiinterface");
   store = gtk_list_store_new(1, G_TYPE_STRING);
   gtk_combo_box_set_model(GTK_COMBO_BOX(w), GTK_TREE_MODEL(store));
@@ -253,6 +253,13 @@ struct gui *gui_open(struct editor *editor)
   gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(w), renderer, TRUE);
   gtk_cell_layout_set_attributes(GTK_CELL_LAYOUT(w), renderer, "text", 0, NULL);
 
+  w = glade_xml_get_widget(gui->xml, "combobox_receive");
+  store = gtk_list_store_new(2, G_TYPE_STRING, G_TYPE_INT);
+  gtk_combo_box_set_model(GTK_COMBO_BOX(w), GTK_TREE_MODEL(store));
+  renderer = gtk_cell_renderer_text_new();
+  gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(w), renderer, TRUE);
+  gtk_cell_layout_set_attributes(GTK_CELL_LAYOUT(w), renderer, "text", 0, NULL);
+
   /* Set default path in file dialogs */
   songpath = editor_song_path_get(gui->editor);
   if (songpath != NULL) {
@@ -369,7 +376,6 @@ void gui_refresh(struct gui *gui)
     gui->refresh_mask ^= GUI_REFRESH_INSTRUMENT;
   }
   g_mutex_unlock(gui->refresh_mutex);
-  g_thread_yield();
 
   g_mutex_lock(gui->refresh_mutex);
   if (gui->refresh_mask & GUI_REFRESH_TEMPO) {
@@ -377,7 +383,6 @@ void gui_refresh(struct gui *gui)
     gui->refresh_mask ^= GUI_REFRESH_TEMPO;
   }
   g_mutex_unlock(gui->refresh_mutex);
-  g_thread_yield();
 
   g_mutex_lock(gui->refresh_mutex);
   if (gui->refresh_mask & GUI_REFRESH_INFO) {
@@ -385,7 +390,6 @@ void gui_refresh(struct gui *gui)
     gui->refresh_mask ^= GUI_REFRESH_INFO;
   }
   g_mutex_unlock(gui->refresh_mutex);
-  g_thread_yield();
 
   g_mutex_lock(gui->refresh_mutex);
   if (gui->refresh_mask & GUI_REFRESH_WINDOW_TITLE) {
@@ -393,7 +397,6 @@ void gui_refresh(struct gui *gui)
     gui->refresh_mask ^= GUI_REFRESH_WINDOW_TITLE;
   }
   g_mutex_unlock(gui->refresh_mutex);
-  g_thread_yield();
 
   g_mutex_lock(gui->refresh_mutex);
   if (gui->refresh_mask & GUI_REFRESH_STATUSBAR) {
@@ -401,7 +404,6 @@ void gui_refresh(struct gui *gui)
     gui->refresh_mask ^= GUI_REFRESH_STATUSBAR;
   }
   g_mutex_unlock(gui->refresh_mutex);
-  g_thread_yield();
 
   g_mutex_lock(gui->refresh_mutex);
   if (gui->refresh_mask & GUI_REFRESH_TIMER) {
@@ -409,7 +411,6 @@ void gui_refresh(struct gui *gui)
     gui->refresh_mask ^= GUI_REFRESH_TIMER;
   }
   g_mutex_unlock(gui->refresh_mutex);
-  g_thread_yield();
 
   g_mutex_lock(gui->refresh_mutex);
   if (gui->refresh_mask & GUI_REFRESH_BLOCKLIST) {
@@ -417,7 +418,6 @@ void gui_refresh(struct gui *gui)
     gui->refresh_mask ^= GUI_REFRESH_BLOCKLIST;
   }
   g_mutex_unlock(gui->refresh_mutex);
-  g_thread_yield();
 
   g_mutex_lock(gui->refresh_mutex);
   if (gui->refresh_mask & GUI_REFRESH_PLAYSEQ) {
@@ -425,7 +425,6 @@ void gui_refresh(struct gui *gui)
     gui->refresh_mask ^= GUI_REFRESH_PLAYSEQ;
   }
   g_mutex_unlock(gui->refresh_mutex);
-  g_thread_yield();
 
   g_mutex_lock(gui->refresh_mutex);
   if (gui->refresh_mask & GUI_REFRESH_PLAYSEQ_RECREATE) {
@@ -433,7 +432,6 @@ void gui_refresh(struct gui *gui)
     gui->refresh_mask ^= GUI_REFRESH_PLAYSEQ_RECREATE;
   }
   g_mutex_unlock(gui->refresh_mutex);
-  g_thread_yield();
 
   g_mutex_lock(gui->refresh_mutex);
   if (gui->refresh_mask & GUI_REFRESH_PLAYSEQLIST) {
@@ -441,7 +439,6 @@ void gui_refresh(struct gui *gui)
     gui->refresh_mask ^= GUI_REFRESH_PLAYSEQLIST;
   }
   g_mutex_unlock(gui->refresh_mutex);
-  g_thread_yield();
 
   g_mutex_lock(gui->refresh_mutex);
   if (gui->refresh_mask & GUI_REFRESH_SECTIONLIST) {
@@ -449,7 +446,6 @@ void gui_refresh(struct gui *gui)
     gui->refresh_mask ^= GUI_REFRESH_SECTIONLIST;
   }
   g_mutex_unlock(gui->refresh_mutex);
-  g_thread_yield();
 
   g_mutex_lock(gui->refresh_mutex);
   if (gui->refresh_mask & GUI_REFRESH_MESSAGELIST) {
@@ -457,7 +453,6 @@ void gui_refresh(struct gui *gui)
     gui->refresh_mask ^= GUI_REFRESH_MESSAGELIST;
   }
   g_mutex_unlock(gui->refresh_mutex);
-  g_thread_yield();
 
   g_mutex_lock(gui->refresh_mutex);
   if (gui->refresh_mask & GUI_REFRESH_TRACKVOLUMES) {
@@ -465,7 +460,6 @@ void gui_refresh(struct gui *gui)
     gui->refresh_mask ^= GUI_REFRESH_TRACKVOLUMES;
   }
   g_mutex_unlock(gui->refresh_mutex);
-  g_thread_yield();
 
   g_mutex_lock(gui->refresh_mutex);
   if (gui->refresh_mask & GUI_REFRESH_SONG) {
@@ -473,7 +467,6 @@ void gui_refresh(struct gui *gui)
     gui->refresh_mask ^= GUI_REFRESH_SONG;
   }
   g_mutex_unlock(gui->refresh_mutex);
-  g_thread_yield();
 
   g_mutex_lock(gui->refresh_mutex);
   if (gui->refresh_mask & GUI_REFRESH_TRACKER) {
@@ -481,7 +474,6 @@ void gui_refresh(struct gui *gui)
     gui->refresh_mask ^= GUI_REFRESH_TRACKER;
   }
   g_mutex_unlock(gui->refresh_mutex);
-  g_thread_yield();
 }
 
 /* Refresh the tracker */
@@ -534,8 +526,6 @@ void gui_song_refresh(struct gui *gui)
 /* Refresh widgets related to the properties of the current instrument */
 void gui_instrument_refresh(struct gui *gui)
 {
-  char instrument[3];
-  snprintf(instrument, 3, "%.2d", gui->editor->instrument + 1);
   GtkSpinButton *spinbutton = GTK_SPIN_BUTTON(glade_xml_get_widget(gui->xml, "spinbutton_instrument"));
   GtkEntry *entry = GTK_ENTRY(glade_xml_get_widget(gui->xml, "entry_instrumentproperties_name"));
   GtkComboBox *combobox = GTK_COMBO_BOX(glade_xml_get_widget(gui->xml, "combobox_instrumentproperties_midiinterface"));
@@ -543,9 +533,9 @@ void gui_instrument_refresh(struct gui *gui)
   GtkRange *range_transpose = GTK_RANGE(glade_xml_get_widget(gui->xml, "scale_instrumentproperties_transpose"));
   GtkRange *range_hold = GTK_RANGE(glade_xml_get_widget(gui->xml, "scale_instrumentproperties_hold"));
   GtkRange *range_midich = GTK_RANGE(glade_xml_get_widget(gui->xml, "scale_instrumentproperties_midich"));
-  char *name;
+  char midiinterface[256], *name;
   int i;
-
+  
   /* Refresh the main window widgets */
   g_signal_handlers_block_by_func(spinbutton, on_spinbutton_instrument_changed, gui);
   g_signal_handlers_block_by_func(entry, on_entry_instrumentproperties_name_changed, gui);
@@ -572,8 +562,14 @@ void gui_instrument_refresh(struct gui *gui)
   gtk_list_store_clear(store);
   for (i = 0; i < gui->editor->nummidiinterfaces; i++) {
     GtkTreeIter iter;
+
+    if (gui->editor->midiinterfaces[i]->devicename != NULL)
+      snprintf(midiinterface, 256, "%s (%s)", gui->editor->midiinterfaces[i]->name, gui->editor->midiinterfaces[i]->devicename);
+    else
+      snprintf(midiinterface, 256, "%s", gui->editor->midiinterfaces[i]->name);
+
     gtk_list_store_append(store, &iter);
-    gtk_list_store_set(store, &iter, 0, gui->editor->midiinterfaces[i]->name, -1);
+    gtk_list_store_set(store, &iter, 0, midiinterface, -1);
   }
   gtk_combo_box_set_active(combobox, editor_song_instrument_current_midiinterface_get(gui->editor));
   gtk_adjustment_set_value(gtk_range_get_adjustment(range_volume), editor_song_instrument_current_defaultvelocity_get(gui->editor));
@@ -928,7 +924,7 @@ void gui_trackvolumes_refresh(struct gui *gui)
   GtkWidget *table = glade_xml_get_widget(gui->xml, "table_trackvolumes");
   int n = editor_song_maxtracks_get(gui->editor);
   char track[10];
-  
+
   /* Destroy all children */
   gtk_container_foreach(GTK_CONTAINER(table), (GtkCallback) gtk_widget_destroy, NULL);
   gtk_table_resize(GTK_TABLE(table), 5, n);
@@ -942,13 +938,13 @@ void gui_trackvolumes_refresh(struct gui *gui)
     GtkWidget *w, *x;
 
     /* Track numbers */
-    snprintf(track, 10, "Track %d", i);
+    snprintf(track, 10, "Track %d", i + 1);
     x = gtk_label_new(track);
     gtk_table_attach(GTK_TABLE(table), x, i, i + 1, 0, 1, GTK_EXPAND | GTK_FILL, GTK_FILL, 2, 4);
-    
+
     /* Volume sliders */
     w = (GtkWidget *) gtk_adjustment_new(volume, 0, 127, 1, 8, 0);
-    g_object_set_data(G_OBJECT(w), "track", (gpointer)i);
+    g_object_set_data(G_OBJECT(w), "track", (gpointer) i);
     g_signal_connect(G_OBJECT(w), "value_changed", G_CALLBACK(on_adjustment_trackvolumes_volume_changed), gui);
     x = gtk_vscale_new(GTK_ADJUSTMENT(w));
     gtk_range_set_inverted(GTK_RANGE(x), TRUE);
@@ -957,7 +953,7 @@ void gui_trackvolumes_refresh(struct gui *gui)
 
     /* Mute buttons */
     w = gtk_check_button_new_with_label("Mute");
-    g_object_set_data(G_OBJECT(w), "track", (gpointer)i);
+    g_object_set_data(G_OBJECT(w), "track", (gpointer) i);
     if (mute)
       gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w), TRUE);
     g_signal_connect(G_OBJECT(w), "toggled", G_CALLBACK(on_togglebutton_trackvolumes_mute_toggled), gui);
@@ -965,7 +961,7 @@ void gui_trackvolumes_refresh(struct gui *gui)
 
     /* Solo buttons */
     w = gtk_check_button_new_with_label("Solo");
-    g_object_set_data(G_OBJECT(w), "track", (gpointer)i);
+    g_object_set_data(G_OBJECT(w), "track", (gpointer) i);
     if (solo)
       gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w), TRUE);
     g_signal_connect(G_OBJECT(w), "toggled", G_CALLBACK(on_togglebutton_trackvolumes_solo_toggled), gui);
@@ -974,7 +970,7 @@ void gui_trackvolumes_refresh(struct gui *gui)
     /* Track names */
     x = gtk_entry_new();
     gtk_entry_set_width_chars(GTK_ENTRY(x), 7);
-    g_object_set_data(G_OBJECT(x), "track", (gpointer)i);
+    g_object_set_data(G_OBJECT(x), "track", (gpointer) i);
     g_signal_connect(G_OBJECT(x), "changed", G_CALLBACK(on_entry_trackvolumes_name_changed), gui);
     if (name != NULL)
       gtk_entry_set_text(GTK_ENTRY(x), name);
@@ -1025,3 +1021,54 @@ void gui_preferences_refresh(struct gui *gui)
   g_signal_handlers_unblock_by_func(treeview, on_cell_preferences_midi_interfaces_toggled, gui);
   g_signal_handlers_unblock_by_func(combobox, on_combobox_preferences_scheduling_mode_changed, gui);
 }
+
+void gui_receive_refresh(struct gui *gui, unsigned int bytes, unsigned int total, gboolean recreate)
+{
+  GtkComboBox *combobox = GTK_COMBO_BOX(glade_xml_get_widget(gui->xml, "combobox_receive"));
+  GtkListStore *store = GTK_LIST_STORE(gtk_combo_box_get_model(combobox));
+  GtkProgressBar *pbar = GTK_PROGRESS_BAR(glade_xml_get_widget(gui->xml, "progressbar_receive"));
+  int i, n = 0;
+  char b[40], midiinterface[256];
+
+  /* Recreate the MIDI interface combo box */
+  if (recreate) {
+    gtk_list_store_clear(store);
+    for (i = 1; i < gui->editor->nummidiinterfaces; i++) {
+      /* Add all inbound MIDI interfaces */
+      if (gui->editor->midiinterfaces[i]->direction & DIRECTION_IN) {
+	GtkTreeIter iter;
+
+	if (gui->editor->midiinterfaces[i]->devicename != NULL)
+	  snprintf(midiinterface, 256, "%s (%s)", gui->editor->midiinterfaces[i]->name, gui->editor->midiinterfaces[i]->devicename);
+	else
+	  snprintf(midiinterface, 256, "%s", gui->editor->midiinterfaces[i]->name);
+
+	gtk_list_store_append(store, &iter);
+	gtk_list_store_set(store, &iter, 0, midiinterface, 1, i, -1);
+	n++;
+      }
+    }
+
+    /* Receiving is only possible if at least one inbound MIDI interface found */
+    if (n > 0) {
+      gtk_combo_box_set_active(combobox, 0);
+      gtk_widget_set_sensitive(glade_xml_get_widget(gui->xml, "button_receive"), TRUE);
+    } else {
+      gtk_widget_set_sensitive(glade_xml_get_widget(gui->xml, "button_receive"), FALSE);
+      gui->editor->receivemidiinterface = -1;
+    }
+  }
+
+  /* Update bytes received count: if total is 0 it is unknown and not shown */
+  if (total == 0) {
+    snprintf(b, 40, "Bytes received: %d", bytes);
+    if (!recreate)
+      gtk_progress_bar_pulse(pbar);
+  } else {
+    snprintf(b, 40, "Bytes received: %d/%d", bytes, total);
+    if (!recreate)
+      gtk_progress_bar_set_fraction(pbar, (gdouble) bytes / total);
+  }
+
+  gtk_progress_bar_set_text(pbar, b);
+}
