@@ -1,6 +1,6 @@
 /* keyboard.c
  *
- * Copyright 2002-2004 Vesa Halttunen
+ * Copyright 2002-2005 Vesa Halttunen
  *
  * This file is part of Tutka.
  *
@@ -26,11 +26,10 @@
 #include <string.h>
 #include <gnome.h>
 #include <glade/glade.h>
-
+#include "tutka.h"
 #include "gui.h"
 #include "callbacks.h"
 #include "editor.h"
-#include "song.h"
 #include "player.h"
 #include "trackerwidget.h"
 
@@ -52,7 +51,6 @@ gboolean keyboard_event(GtkWidget * widget, GdkEventKey * event, gboolean presse
   Tracker *tracker;
   int shift, ctrl, alt;
   struct editor *editor = gui->editor;
-  struct song *song = editor->song;
   gboolean handled = FALSE;
 
   if (GTK_IS_ENTRY(GTK_WINDOW(widget)->focus_widget))
@@ -76,7 +74,7 @@ gboolean keyboard_event(GtkWidget * widget, GdkEventKey * event, gboolean presse
 	break;
       case 'k':
 	/* CTRL-K: Clear until the end of the track */
-	block_clear(song->blocks[editor->block], tracker->cursor_ch, tracker->patpos, tracker->cursor_ch, song->blocks[editor->block]->length - 1);
+	editor_song_block_current_clear(editor, tracker->cursor_ch, tracker->patpos, tracker->cursor_ch, editor_song_block_current_length_get(editor) - 1);
 	tracker_redraw(tracker);
 	handled = TRUE;
 	break;
@@ -86,7 +84,7 @@ gboolean keyboard_event(GtkWidget * widget, GdkEventKey * event, gboolean presse
 	  editor->instrument--;
 
 	  /* Make sure the instrument exists */
-	  song_check_instrument(song, editor->instrument);
+	  editor_song_instrument_current_check(editor);
 	  gui_instrument_refresh(gui);
 	}
 	handled = TRUE;
@@ -96,7 +94,7 @@ gboolean keyboard_event(GtkWidget * widget, GdkEventKey * event, gboolean presse
 	editor->instrument++;
 
 	/* Make sure the instrument exists */
-	song_check_instrument(song, editor->instrument);
+	editor_song_instrument_current_check(editor);
 	gui_instrument_refresh(gui);
 
 	handled = TRUE;
@@ -109,16 +107,18 @@ gboolean keyboard_event(GtkWidget * widget, GdkEventKey * event, gboolean presse
 	break;
       case GDK_Delete:
 	{
-	  int i;
-
-	  /* Delete command and refresh */
-	  for (i = 0; i < song->blocks[editor->block]->commandpages; i++)
-	    block_set_command_full(song->blocks[editor->block], editor->line, tracker->cursor_ch, i, 0, 0);
-	  tracker_redraw_row(tracker, editor->line);
-
-	  editor_set_line(editor, editor->line + editor->space);
-
-	  handled = TRUE;
+          if (widget == gui->window_main) {
+            int i, n = editor_song_block_current_commandpages_get(editor);
+  
+            /* Delete command and refresh */
+            for (i = 0; i < n; i++)
+              editor_song_block_current_command_full_set(editor, editor->line, tracker->cursor_ch, i, 0, 0);
+            tracker_redraw_row(tracker, editor->line);
+  
+            editor_set_line(editor, editor->line + editor->space);
+  
+            handled = TRUE;
+          }
 	  break;
 	}
       case '0':
@@ -132,17 +132,32 @@ gboolean keyboard_event(GtkWidget * widget, GdkEventKey * event, gboolean presse
       case '8':
       case '9':
 	{
-	  /* CTRL-0-9: Set space value */
-	  GtkSpinButton *spinbutton = GTK_SPIN_BUTTON(glade_xml_get_widget(gui->xml, "spinbutton_space"));
+	  if (alt) {
+	    /* CTRL+Alt+0-9: Solo tracks */
+	    int channel = 9;
+	    if (event->keyval >= '1' && event->keyval <= '9')
+	      channel = event->keyval - '1';
+	    if (channel < editor_song_maxtracks_get(editor)) {
+	      unsigned int mute = editor_song_track_solo_get(editor, channel);
+	      if (mute)
+		editor_song_track_solo_set(editor, channel, 0);
+	      else
+		editor_song_track_solo_set(editor, channel, 1);
+	      gui_trackvolumes_refresh(gui);
+	    }
+	  } else {
+	    /* CTRL+0-9: Set space value */
+	    GtkSpinButton *spinbutton = GTK_SPIN_BUTTON(glade_xml_get_widget(gui->xml, "spinbutton_space"));
 
-	  editor->space = event->keyval - '0';
+	    editor->space = event->keyval - '0';
 
-	  g_signal_handlers_block_by_func(spinbutton, on_spinbutton_space_changed, gui);
-	  gtk_spin_button_set_value(spinbutton, editor->space);
-	  g_signal_handlers_unblock_by_func(spinbutton, on_spinbutton_space_changed, gui);
+	    g_signal_handlers_block_by_func(spinbutton, on_spinbutton_space_changed, gui);
+	    gtk_spin_button_set_value(spinbutton, editor->space);
+	    g_signal_handlers_unblock_by_func(spinbutton, on_spinbutton_space_changed, gui);
 
-	  handled = TRUE;
-	  break;
+	    handled = TRUE;
+	    break;
+	  }
 	}
       }
     } else if (shift) {
@@ -155,59 +170,50 @@ gboolean keyboard_event(GtkWidget * widget, GdkEventKey * event, gboolean presse
 	break;
       case GDK_Right:
 	/* Shift-Right: Previous position */
-	if (editor->position < song->playseqs[editor->playseq]->length - 1)
+	if (editor->position < editor_song_playseq_current_length_get(editor) - 1)
 	  editor_set_position(editor, editor->position + 1);
 	handled = TRUE;
 	break;
       case GDK_ISO_Left_Tab:
-	/* Shift-Tab: Previous track */
-	if ((tracker)->cursor_item > 0) {
-	  (tracker)->cursor_item = 0;
-	  tracker_step_cursor_channel(tracker, 0);
-	} else
-	  tracker_step_cursor_channel(tracker, -1);
-	handled = TRUE;
+	/* Shift-Tab: Previous track - only in main window */
+	if (widget == gui->window_main) {
+	  if ((tracker)->cursor_item > 0) {
+	    (tracker)->cursor_item = 0;
+	    tracker_step_cursor_channel(tracker, 0);
+	  } else
+	    tracker_step_cursor_channel(tracker, -1);
+	  handled = TRUE;
+	}
 	break;
       case GDK_Delete:
 	{
-	  int i;
-
-	  /* Delete note+command and refresh */
-	  block_set_note(song->blocks[editor->block], editor->line, tracker->cursor_ch, 0, 0, 0);
-	  for (i = 0; i < song->blocks[editor->block]->commandpages; i++)
-	    block_set_command_full(song->blocks[editor->block], editor->line, tracker->cursor_ch, i, 0, 0);
-	  tracker_redraw_row(tracker, editor->line);
-
-	  editor_set_line(editor, editor->line + editor->space);
-
-	  handled = TRUE;
+          if (widget == gui->window_main) {
+            int i, n = editor_song_block_current_commandpages_get(editor);
+  
+            /* Delete note+command and refresh */
+            editor_song_block_current_note_set(editor, editor->line, tracker->cursor_ch, 0, 0, 0);
+            for (i = 0; i < n; i++)
+              editor_song_block_current_command_full_set(editor, editor->line, tracker->cursor_ch, i, 0, 0);
+            tracker_redraw_row(tracker, editor->line);
+  
+            editor_set_line(editor, editor->line + editor->space);
+  
+            handled = TRUE;
+          }
 	  break;
 	}
       case GDK_BackSpace:
 	/* Shift-Backspace: Insert row */
-	if (editor->edit) {
-	  int i;
-	  struct block *b;
-	  if (alt) {
-	    int j;
-	    b = block_copy(song->blocks[editor->block], 0, tracker->patpos, song->blocks[editor->block]->tracks - 1, song->blocks[editor->block]->length - 1);
-	    block_paste(song->blocks[editor->block], b, 0, tracker->patpos + 1);
-	    for (j = 0; j < song->blocks[editor->block]->tracks; j++) {
-	      block_set_note(song->blocks[editor->block], tracker->patpos, j, 0, 0, 0);
-	      for (i = 0; i < song->blocks[editor->block]->commandpages; i++)
-		block_set_command_full(song->blocks[editor->block], tracker->patpos, j, i, 0, 0);
-	    }
-	  } else {
-	    b = block_copy(song->blocks[editor->block], tracker->cursor_ch, tracker->patpos, tracker->cursor_ch, song->blocks[editor->block]->length - 1);
-	    block_paste(song->blocks[editor->block], b, tracker->cursor_ch, tracker->patpos + 1);
-	    block_set_note(song->blocks[editor->block], tracker->patpos, tracker->cursor_ch, 0, 0, 0);
-	    for (i = 0; i < song->blocks[editor->block]->commandpages; i++)
-	      block_set_command_full(song->blocks[editor->block], tracker->patpos, tracker->cursor_ch, i, 0, 0);
-	  }
-	  block_free(b);
-	  tracker_redraw(tracker);
-	}
-	handled = TRUE;
+	if (widget == gui->window_main) {
+          if (editor->edit) {
+            if (alt)
+              editor_song_block_current_insert_line_track_all(editor);
+            else  
+              editor_song_block_current_insert_line(editor, tracker->cursor_ch);
+            tracker_redraw(tracker);
+          }
+          handled = TRUE;
+        }
 	break;
       default:
 	break;
@@ -234,37 +240,27 @@ gboolean keyboard_event(GtkWidget * widget, GdkEventKey * event, gboolean presse
       case '8':
       case '9':
       case '0':{
-	  /* Alt+0-9: Switch channels on/off */
+	  /* Alt+0-9: Mute tracks */
 	  int channel = 9;
 	  if (event->keyval >= '1' && event->keyval <= '9')
 	    channel = event->keyval - '1';
-	  if (channel < editor->song->maxtracks) {
-	    if (editor->song->trackvolumes[channel] > 127)
-	      editor->song->trackvolumes[channel] &= 127;
+	  if (channel < editor_song_maxtracks_get(editor)) {
+	    unsigned int mute = editor_song_track_mute_get(editor, channel);
+	    if (mute)
+	      editor_song_track_mute_set(editor, channel, 0);
 	    else
-	      editor->song->trackvolumes[channel] |= 128;
+	      editor_song_track_mute_set(editor, channel, 1);
 	    gui_trackvolumes_refresh(gui);
 	  }
 	  break;
 	}
       case GDK_BackSpace:
-	{
-	  /* Backspace: delete line */
-	  int i, j;
-	  struct block *b = block_copy(song->blocks[editor->block],
-				       0, tracker->patpos + 1,
-				       song->blocks[editor->block]->tracks - 1,
-				       song->blocks[editor->block]->length - 1);
-	  block_paste(song->blocks[editor->block], b, 0, tracker->patpos);
-	  block_free(b);
-	  for (i = 0; i < song->blocks[editor->block]->tracks; i++) {
-	    block_set_note(song->blocks[editor->block], song->blocks[editor->block]->length - 1, i, 0, 0, 0);
-	    for (j = 0; j < song->blocks[editor->block]->commandpages; j++)
-	      block_set_command_full(song->blocks[editor->block], song->blocks[editor->block]->length - 1, i, j, 0, 0);
-	  }
-	  tracker_redraw(tracker);
-	}
-	handled = TRUE;
+	/* Backspace: delete line */
+	if (widget == gui->window_main) {
+          editor_song_block_current_delete_line_track_all(editor);
+          tracker_redraw(tracker);
+          handled = TRUE;
+        }
 	break;
       default:
 	break;
@@ -400,19 +396,19 @@ gboolean keyboard_event(GtkWidget * widget, GdkEventKey * event, gboolean presse
 
 	      if (editor->edit) {
 		/* Set note and refresh */
-		block_set_note(song->blocks[editor->block], editor->line, tracker->cursor_ch, editor->octave, data, editor->instrument + 1);
+		editor_song_block_current_note_set(editor, editor->line, tracker->cursor_ch, editor->octave, data, editor->instrument + 1);
 		tracker_redraw_row(tracker, editor->line);
 	      }
 
-              tracker_step_cursor_channel(tracker, 1);
+	      tracker_step_cursor_channel(tracker, 1);
 	      editor->chordstatus++;
-            }
-          } else if (editor->edit) {
-            /* Set note and refresh */
-	    block_set_note(song->blocks[editor->block], editor->line, tracker->cursor_ch, editor->octave, data, editor->instrument + 1);
+	    }
+	  } else if (editor->edit) {
+	    /* Set note and refresh */
+	    editor_song_block_current_note_set(editor, editor->line, tracker->cursor_ch, editor->octave, data, editor->instrument + 1);
 	    tracker_redraw_row(tracker, editor->line);
 	    editor_set_line(editor, editor->line + editor->space);
-          }
+	  }
 	  handled = TRUE;
 	}
       } else if (editor->edit) {
@@ -428,7 +424,7 @@ gboolean keyboard_event(GtkWidget * widget, GdkEventKey * event, gboolean presse
 
 	  if (data != -1) {
 	    /* Set instrument and refresh */
-	    block_set_instrument(song->blocks[editor->block], editor->line, tracker->cursor_ch, data);
+	    editor_song_block_current_instrument_set(editor, editor->line, tracker->cursor_ch, data);
 	    tracker_redraw_row(tracker, editor->line);
 	    editor_set_line(editor, editor->line + editor->space);
 	    handled = TRUE;
@@ -448,7 +444,7 @@ gboolean keyboard_event(GtkWidget * widget, GdkEventKey * event, gboolean presse
 
 	  if (data != -1) {
 	    /* Set effect and refresh */
-	    block_set_command(song->blocks[editor->block], editor->line, tracker->cursor_ch, editor->cmdpage, tracker->cursor_item - 2, data);
+	    editor_song_block_current_command_set(editor, editor->line, tracker->cursor_ch, editor->cmdpage, tracker->cursor_item - 2, data);
 	    tracker_redraw_row(tracker, editor->line);
 	    editor_set_line(editor, editor->line + editor->space);
 	    handled = TRUE;
@@ -460,20 +456,20 @@ gboolean keyboard_event(GtkWidget * widget, GdkEventKey * event, gboolean presse
       switch (event->keyval) {
       case GDK_Shift_R:
 	/* Right shift: Play block */
-	player_start(editor->player, MODE_PLAY_BLOCK, editor->section, editor->position, editor->block, 0);
+	editor_player_start(editor, MODE_PLAY_BLOCK, 0);
 	gui_statusbar_refresh(gui);
 	handled = TRUE;
 	break;
       case GDK_Control_R:
 	/* Right Control: Play song */
-	player_start(editor->player, MODE_PLAY_SONG, editor->section, editor->position, editor->block, 0);
+	editor_player_start(editor, MODE_PLAY_SONG, 0);
 	gui_statusbar_refresh(gui);
 	handled = TRUE;
 	break;
       case GDK_space:
 	/* If the song is playing, stop */
-	if (editor->player->mode != MODE_IDLE) {
-	  player_stop(editor->player);
+	if (editor_player_mode_get(editor) != MODE_IDLE) {
+	  editor_player_stop(editor);
 	  gui_info_refresh(gui);
 	} else {
 	  /* Otherwise toggle edit mode */
@@ -500,13 +496,13 @@ gboolean keyboard_event(GtkWidget * widget, GdkEventKey * event, gboolean presse
       case GDK_F11:
 	{
 	  /* Set active keyboard octave */
-	  GtkOptionMenu *optionmenu = GTK_OPTION_MENU(glade_xml_get_widget(gui->xml, "optionmenu_keyboard"));
+	  GtkComboBox *combobox = GTK_COMBO_BOX(glade_xml_get_widget(gui->xml, "combobox_keyboard"));
 
 	  editor->octave = event->keyval - GDK_F1;
 
-	  g_signal_handlers_block_by_func(optionmenu, on_optionmenu_keyboard_changed, gui);
-	  gtk_option_menu_set_history(optionmenu, event->keyval - GDK_F1);
-	  g_signal_handlers_unblock_by_func(optionmenu, on_optionmenu_keyboard_changed, gui);
+	  g_signal_handlers_block_by_func(combobox, on_combobox_keyboard_changed, gui);
+	  gtk_combo_box_set_active(combobox, event->keyval - GDK_F1);
+	  g_signal_handlers_unblock_by_func(combobox, on_combobox_keyboard_changed, gui);
 
 	  handled = TRUE;
 	  break;
@@ -672,68 +668,76 @@ gboolean keyboard_event(GtkWidget * widget, GdkEventKey * event, gboolean presse
 	note = 30;
 	break;
       case GDK_Down:
-	/* Down: Go down */
-	editor_set_line(editor, editor->line + 1);
-	handled = TRUE;
+	/* Down: Go down -*/
+	if (widget == gui->window_main) {
+          editor_set_line(editor, editor->line + 1);
+          handled = TRUE;
+        }
 	break;
       case GDK_Up:
 	/* Up: Go up */
-	editor_set_line(editor, editor->line - 1);
-	handled = TRUE;
+	if (widget == gui->window_main) {
+          editor_set_line(editor, editor->line - 1);
+          handled = TRUE;
+        }
 	break;
       case GDK_Left:
 	/* Left: Go left */
-	tracker_step_cursor_item(tracker, -1);
-	handled = TRUE;
+	if (widget == gui->window_main) {
+          tracker_step_cursor_item(tracker, -1);
+          handled = TRUE;
+        }
 	break;
       case GDK_Right:
 	/* Right: Go right */
-	tracker_step_cursor_item(tracker, 1);
-	handled = TRUE;
+	if (widget == gui->window_main) {
+          tracker_step_cursor_item(tracker, 1);
+          handled = TRUE;
+        }
 	break;
       case GDK_Tab:
 	/* Tab: Next track */
-	(tracker)->cursor_item = 0;
-	tracker_step_cursor_channel(tracker, 1);
-	handled = TRUE;
+	if (widget == gui->window_main) {
+          (tracker)->cursor_item = 0;
+          tracker_step_cursor_channel(tracker, 1);
+          handled = TRUE;
+        }
 	break;
       case GDK_Home:
 	/* End: Go to the beginning of block */
-	editor_set_line(editor, 0);
-	handled = TRUE;
+	if (widget == gui->window_main) {
+          editor_set_line(editor, 0);
+          handled = TRUE;
+        }
 	break;
       case GDK_End:
 	/* End: Go to the end of block */
-	editor_set_line(editor, song->blocks[editor->block]->length - 1);
-	handled = TRUE;
+	if (widget == gui->window_main) {
+          editor_set_line(editor, editor_song_block_current_length_get(editor) - 1);
+          handled = TRUE;
+        }
 	break;
       case GDK_Page_Down:
 	/* Page down: Go down 8 lines */
-	editor_set_line(editor, editor->line + 8);
-	handled = TRUE;
+	if (widget == gui->window_main) {
+          editor_set_line(editor, editor->line + 8);
+          handled = TRUE;
+        }
 	break;
       case GDK_Page_Up:
 	/* Page up: Go up 8 lines */
-	editor_set_line(editor, editor->line - 8);
-	handled = TRUE;
+	if (widget == gui->window_main) {
+          editor_set_line(editor, editor->line - 8);
+          handled = TRUE;
+        }
 	break;
       case GDK_BackSpace:
-	{
-	  /* Backspace: delete line */
-	  int i;
-	  struct block *b = block_copy(song->blocks[editor->block],
-				       tracker->cursor_ch,
-				       tracker->patpos + 1,
-				       tracker->cursor_ch,
-				       song->blocks[editor->block]->length - 1);
-	  block_paste(song->blocks[editor->block], b, tracker->cursor_ch, tracker->patpos);
-	  block_free(b);
-	  block_set_note(song->blocks[editor->block], song->blocks[editor->block]->length - 1, tracker->cursor_ch, 0, 0, 0);
-	  for (i = 0; i < song->blocks[editor->block]->commandpages; i++)
-	    block_set_command_full(song->blocks[editor->block], song->blocks[editor->block]->length - 1, tracker->cursor_ch, i, 0, 0);
-	  tracker_redraw(tracker);
-	}
-	handled = TRUE;
+	/* Backspace: delete line */
+	if (widget == gui->window_main) {
+          editor_song_block_current_delete_line(editor, tracker->cursor_ch);
+          tracker_redraw(tracker);
+          handled = TRUE;
+        }
 	break;
       }
 
@@ -742,14 +746,14 @@ gboolean keyboard_event(GtkWidget * widget, GdkEventKey * event, gboolean presse
 	editor->instrument = instrument;
 
 	/* Make sure the instrument exists */
-	song_check_instrument(song, instrument);
+	editor_song_instrument_current_check(editor);
 	gui_instrument_refresh(gui);
 	handled = TRUE;
       }
 
       /* Play note if a key was pressed but not if cursor is in cmd pos */
       if (note != -1 && tracker->cursor_item == 0) {
-	player_play_note(editor->player, editor->instrument, editor->octave * 12 + note, 127, tracker->cursor_ch);
+	editor_player_play_note(editor, note, tracker->cursor_ch);
 	handled = TRUE;
       }
     }
