@@ -29,6 +29,7 @@
 #include <glib.h>
 #include "editor.h"
 #include "gui.h"
+#include "callbacks.h"
 #include "conversion.h"
 
 extern gboolean gui_quit;
@@ -68,13 +69,25 @@ int main(int argc, char *argv[])
       while (gtk_events_pending())
 	gtk_main_iteration_do(FALSE);
 
-      /* Read MIDI input if there is any pending */
-      for (i = 0; i < editor->nummidiinterfaces; i++)
-        while ((data = midi_read_raw(editor->midiinterfaces[i])) != NULL) {
-          editor_midi_input(editor, data);
-          free(data);
+      if (!editor->receiveactive) {
+        /* Read MIDI input if there is any pending */
+        for (i = 0; i < editor->nummidiinterfaces; i++)
+          while ((data = midi_read_raw(editor->midiinterfaces[i])) != NULL) {
+            editor_midi_input(editor, data);
+            free(data);
+          }
+      } else {
+        /* Receive a MIDI message */
+        if (editor_song_message_receive(editor))
+          on_button_receive_close_clicked(NULL, editor->gui);          
+        else {
+          if (editor->receiveautostop)
+            gui_receive_refresh(editor->gui, editor->receivedbytes, 0, FALSE);
+          else
+            gui_receive_refresh(editor->gui, editor->receivedbytes, editor->song->messages[editor->receivemessage]->length, FALSE);
         }
-
+      }
+        
       /* Refresh the GUI */
       gui_refresh(editor->gui);
 
