@@ -36,29 +36,60 @@ unsigned int midi_get_num_alsa_devices()
     fprintf(stderr, "cannot determine card number: %s", snd_strerror(err));
     return devices;
   }
+
   if (card < 0) {
     fprintf(stderr, "no sound card found");
     return devices;
   }
+
   do {
     int device = -1;
     char devname[32];
     snd_ctl_t *ctl;
 
     snprintf(devname, 32, "hw:%d", card);
+
     if ((err = snd_ctl_open(&ctl, devname, 0)) < 0) {
       fprintf(stderr, "cannot open control for card %d: %s", card, snd_strerror(err));
       return devices;
     }
+
     for (;;) {
+      snd_rawmidi_info_t *info;
+
       if ((err = snd_ctl_rawmidi_next_device(ctl, &device)) < 0) {
 	fprintf(stderr, "cannot determine device number: %s", snd_strerror(err));
 	break;
       }
+
       if (device < 0)
 	break;
-      devices++;
+
+      /* Get device information */
+      snd_rawmidi_info_alloca(&info);
+      snd_rawmidi_info_set_device(info, device);
+      snd_rawmidi_info_set_subdevice(info, 0);
+      snd_rawmidi_info_set_stream(info, SND_RAWMIDI_STREAM_OUTPUT);
+
+      if ((err = snd_ctl_rawmidi_info(ctl, info)) < 0 && err != -ENOENT) {
+	fprintf(stderr, "cannot get rawmidi information %d:%d: %s", card, device, snd_strerror(err));
+	break;
+      }
+
+      if (err == -ENOENT) {
+	snd_rawmidi_info_set_stream(info, SND_RAWMIDI_STREAM_INPUT);
+	if ((err = snd_ctl_rawmidi_info(ctl, info)) < 0 && err != -ENOENT) {
+	  fprintf(stderr, "cannot get rawmidi information %d:%d: %s", card, device, snd_strerror(err));
+	  break;
+	}
+      }
+
+      if (err == -ENOENT)
+	break;
+
+      devices += snd_rawmidi_info_get_subdevices_count(info);
     }
+
     snd_ctl_close(ctl);
 
     if ((err = snd_card_next(&card)) < 0) {
@@ -80,46 +111,83 @@ struct midi **midi_get_alsa_devices()
     fprintf(stderr, "cannot determine card number: %s", snd_strerror(err));
     return midiinterfaces;
   }
+
   if (card < 0) {
     fprintf(stderr, "no sound card found");
     return midiinterfaces;
   }
+
   do {
     snd_ctl_t *ctl;
-    char devname[32];
+    char devname[32], fullname[256];
     int device = -1;
 
     snprintf(devname, 32, "hw:%d", card);
+
     if ((err = snd_ctl_open(&ctl, devname, 0)) < 0) {
       fprintf(stderr, "cannot open control for card %d: %s", card, snd_strerror(err));
       return midiinterfaces;
     }
+
     for (;;) {
+      snd_rawmidi_info_t *info;
+      char *name, *sub_name;
+      int i, n;
+
       if ((err = snd_ctl_rawmidi_next_device(ctl, &device)) < 0) {
 	fprintf(stderr, "cannot determine device number: %s", snd_strerror(err));
 	break;
       }
+
       if (device < 0)
 	break;
 
-      snd_rawmidi_info_t *info;
-      char *name = NULL;
-
+      /* Get device and subdevice information */
       snd_rawmidi_info_alloca(&info);
       snd_rawmidi_info_set_device(info, device);
       snd_rawmidi_info_set_subdevice(info, 0);
       snd_rawmidi_info_set_stream(info, SND_RAWMIDI_STREAM_OUTPUT);
-      if ((err = snd_ctl_rawmidi_info(ctl, info)) < 0 && err != -ENOENT)
+
+      if ((err = snd_ctl_rawmidi_info(ctl, info)) < 0 && err != -ENOENT) {
 	fprintf(stderr, "cannot get rawmidi information %d:%d: %s", card, device, snd_strerror(err));
+	break;
+      }
+
       if (err == -ENOENT) {
 	snd_rawmidi_info_set_stream(info, SND_RAWMIDI_STREAM_INPUT);
-	if ((err = snd_ctl_rawmidi_info(ctl, info)) < 0 && err != -ENOENT)
+	if ((err = snd_ctl_rawmidi_info(ctl, info)) < 0 && err != -ENOENT) {
 	  fprintf(stderr, "cannot get rawmidi information %d:%d: %s", card, device, snd_strerror(err));
+	  break;
+	}
+      }
+
+      if (err == -ENOENT)
+	break;
+
+      name = (char *)snd_rawmidi_info_get_name(info);
+      n = snd_rawmidi_info_get_subdevices_count(info);
+
+      /* Allocate MIDI interfaces */
+      if (n == 1) {
+	snprintf(devname, 32, "hw:%d,%d", card, device);
+	midiinterfaces[current++] = midi_alloc(MIDI_ALSA, devname, name);
+      } else {
+	for (i = 0; i < n; i++) {
+	  snd_rawmidi_info_set_subdevice(info, i);
+	  if ((err = snd_ctl_rawmidi_info(ctl, info)) < 0) {
+	    fprintf(stderr, "cannot get rawmidi information %d:%d:%d: %s", card, device, i, snd_strerror(err));
+	    break;
+	  }
+	  sub_name = (char *)snd_rawmidi_info_get_subdevice_name(info);
+
+	  snprintf(devname, 32, "hw:%d,%d,%d", card, device, i);
+	  if (strlen(sub_name) > 0)
+	    snprintf(fullname, 256, "%s %s", name, sub_name);
+	  else
+	    snprintf(fullname, 256, "%s %d", name, i + 1);
+	  midiinterfaces[current++] = midi_alloc(MIDI_ALSA, devname, fullname);
+	}
       }
-      if (err != -ENOENT)
-	name = (char *)snd_rawmidi_info_get_name(info);
-      snprintf(devname, 32, "hw:%d,%d", card, device);
-      midiinterfaces[current++] = midi_alloc(MIDI_ALSA, devname, name);
     }
     snd_ctl_close(ctl);
 
@@ -161,31 +229,47 @@ void midi_free(struct midi *midi)
 }
 
 /* Opens the MIDI interfaces defined in a string */
-void midi_open_defined(struct midi **midiinterfaces, unsigned int nummidiinterfaces, char *in, char *out)
+void midi_open_defined(struct midi **midiinterfaces, unsigned int nummidiinterfaces, char *in, char *out, char *unavailablein, char *unavailableout)
 {
   nullcheck_void(midiinterfaces, midi_open_defined);
   nullcheck_void(in, midi_open_defined);
   nullcheck_void(out, midi_open_defined);
+  nullcheck_void(unavailablein, midi_open_defined);
+  nullcheck_void(unavailableout, midi_open_defined);
 
   int i, j, last;
-  unsigned int direction;
-  char *current, *next;
+  unsigned int direction = 0;
+  char *current = NULL, *next = NULL;
 
   /* Close all devices in the MIDI interfaces list */
   for (i = 0; i < nummidiinterfaces; i++)
     midi_close(midiinterfaces[i], DIRECTION_BOTH);
 
-  /* Handle both input and output */
-  for (j = 0; j < 2; j++) {
+  /* Handle both input and output, available and unavailable */
+  for (j = 0; j < 4; j++) {
     last = 0;
-    if (j == 0) {
+
+    switch (j) {
+    case 0:
       current = in;
       next = in;
       direction = DIRECTION_IN;
-    } else {
+      break;
+    case 1:
       current = out;
       next = out;
       direction = DIRECTION_OUT;
+      break;
+    case 2:
+      current = unavailablein;
+      next = unavailablein;
+      direction = DIRECTION_IN;
+      break;
+    case 3:
+      current = unavailableout;
+      next = unavailableout;
+      direction = DIRECTION_OUT;
+      break;
     }
 
     while (!last) {
@@ -216,6 +300,7 @@ unsigned int midi_open(struct midi *midi, unsigned int direction)
 {
   nullcheck_int(midi, midi_open);
 
+  unsigned int unavailabledirection = 0;
   int retval = 1;
 
   /* If no direction specified or device is already open do nothing */
@@ -244,6 +329,7 @@ unsigned int midi_open(struct midi *midi, unsigned int direction)
 	if (inerr) {
 	  fprintf(stderr, "Could not open ALSA MIDI device %s for input: %d\n", midi->devicename, inerr);
 	  direction &= (~DIRECTION_IN);
+          unavailabledirection |= DIRECTION_IN;
 	  retval = 0;
 	}
       }
@@ -254,6 +340,7 @@ unsigned int midi_open(struct midi *midi, unsigned int direction)
 	if (outerr) {
 	  fprintf(stderr, "Could not open ALSA MIDI device %s for output: %d\n", midi->devicename, outerr);
 	  direction &= (~DIRECTION_OUT);
+          unavailabledirection |= DIRECTION_OUT;
 	  retval = 0;
 	}
       }
@@ -275,6 +362,7 @@ unsigned int midi_open(struct midi *midi, unsigned int direction)
 	if (oss->in == -1) {
 	  fprintf(stderr, "Could not open OSS MIDI device %s for input.\n", midi->devicename);
 	  direction &= (~DIRECTION_IN);
+          unavailabledirection |= DIRECTION_IN;
 	  retval = 0;
 	}
       }
@@ -283,6 +371,7 @@ unsigned int midi_open(struct midi *midi, unsigned int direction)
 	if (oss->out == -1) {
 	  fprintf(stderr, "Could not open OSS MIDI device %s for output.\n", midi->devicename);
 	  direction &= (~DIRECTION_OUT);
+          unavailabledirection |= DIRECTION_OUT;
 	  retval = 0;
 	}
       }
@@ -294,6 +383,7 @@ unsigned int midi_open(struct midi *midi, unsigned int direction)
 
       if (direction != DIRECTION_BOTH) {
 	fprintf(stderr, "midi_open(): Only bidirectional mode supported for buffers.\n");
+        unavailabledirection |= direction;
 	direction = DIRECTION_NONE;
 	retval = 0;
       } else {
@@ -308,6 +398,7 @@ unsigned int midi_open(struct midi *midi, unsigned int direction)
   }
 
   midi->direction |= direction;
+  midi->unavailabledirection |= unavailabledirection;
 
   /* If both input and output are closed free the data structure */
   if (midi->direction == DIRECTION_NONE) {
@@ -332,7 +423,7 @@ void midi_close(struct midi *midi, unsigned int direction)
     midi->direction = DIRECTION_NONE;
     return;
   }
-  
+
   switch (midi->type) {
   case MIDI_OSS:
     {
@@ -342,11 +433,13 @@ void midi_close(struct midi *midi, unsigned int direction)
       if (direction & DIRECTION_IN && oss->in != -1) {
 	fsync(oss->in);
 	close(oss->in);
+        oss->in = -1;
       }
 
       if (direction & DIRECTION_OUT && oss->out != -1) {
 	fsync(oss->out);
 	close(oss->out);
+        oss->out = -1;
       }
       break;
     }
@@ -355,14 +448,16 @@ void midi_close(struct midi *midi, unsigned int direction)
       struct midi_alsa *alsa = midi->data;
 
       /* Close input and output devices if requested */
-      if (direction & DIRECTION_IN && alsa->in != NULL) {
+      if ((direction & DIRECTION_IN) && alsa->in != NULL) {
 	snd_rawmidi_drain(alsa->in);
 	snd_rawmidi_close(alsa->in);
+        alsa->in = NULL;
       }
 
-      if (direction & DIRECTION_OUT && alsa->out != NULL) {
+      if ((direction & DIRECTION_OUT) && alsa->out != NULL) {
 	snd_rawmidi_drain(alsa->out);
 	snd_rawmidi_close(alsa->out);
+        alsa->out = NULL;
       }
       break;
     }
@@ -371,6 +466,7 @@ void midi_close(struct midi *midi, unsigned int direction)
       struct midi_buffer *buffer = midi->data;
 
       free(buffer->data);
+      buffer->data = NULL;
       break;
     }
   default:
@@ -632,7 +728,7 @@ void midi_write_raw(struct midi *midi, unsigned char *message, unsigned short le
       else
 	break;
     }
-    
+
     /* Allocate buffer */
     newmessage = (unsigned char *)malloc(length + l);
     newmessage[0] = 0xf0;
@@ -668,18 +764,18 @@ void midi_clock(struct midi *midi)
 void midi_tempo(struct midi *midi, unsigned int tempo)
 {
   nullcheck_void(midi, midi_tempo);
-  
+
   unsigned char data[] = { 0xff, 0x51, 0x03, 0x00, 0x00, 0x00 };
   unsigned int ms = 60000000 / tempo;
-  
+
   /* Tempo is only relevant when exporting */
   if (midi->type != MIDI_BUFFER)
     return;
-  
+
   data[3] = (ms >> 16) & 0xff;
   data[4] = (ms >> 8) & 0xff;
   data[5] = ms & 0xff;
-  
+
   midi_write(midi, data, 6);
 }
 
@@ -725,48 +821,45 @@ unsigned char *midi_read_raw(struct midi *midi)
 }
 
 /* Receives a MIDI message */
-void midi_read_system_exclusive(struct midi *midi, struct message *message, int autostop)
+int midi_read_system_exclusive(struct midi *midi, struct message *message, unsigned int read, int autostop)
 {
-  nullcheck_void(midi, midi_read_system_exclusive);
-  nullcheck_void(message, midi_read_system_exclusive);
+  nullcheck_int(midi, midi_read_system_exclusive);
+  nullcheck_int(message, midi_read_system_exclusive);
 
-  int status = 0, i = 0;
+  int status, i = read;
   unsigned char c = 0;
 
   if (autostop == 1) {
     /* Read until 0xf7 (end of SysEx) */
-    c = 0;
-    message->length = 0;
-
     while (c != 0xf7) {
       status = midi_read(midi, &c, 1);
 
-      if (status == 1) {
-	if (i >= message->length) {
-	  message->length += 65536;
-	  message->data = (unsigned char *)realloc(message->data, message->length);
-	}
+      if (status == 0)
+	return i - read;
 
-	if (c != 0xf1 && c != 0xf8 && c != 0xf9 && c != 0xfe)
-	  message->data[i++] = c;
+      if (i >= message->length) {
+	message->length += 65536;
+	message->data = (unsigned char *)realloc(message->data, message->length);
       }
+
+      if (c != 0xf1 && c != 0xf8 && c != 0xf9 && c != 0xfe)
+	message->data[i++] = c;
     }
+
+    return -(i - read);
   } else {
     /* Read until the buffer is full */
     while (i < message->length) {
       status = midi_read(midi, &c, 1);
 
-      if (status == 1) {
-	if (c != 0xf1 && c != 0xf8 && c != 0xf9 && c != 0xfe)
-	  message->data[i++] = c;
-      }
-    }
-  }
+      if (status == 0)
+	return i - read;
 
-  /* Remove unnecessary bytes if necessary */
-  if (i < message->length) {
-    message->length = i;
-    message->data = (unsigned char *)realloc(message->data, message->length);
+      if (c != 0xf1 && c != 0xf8 && c != 0xf9 && c != 0xfe)
+	message->data[i++] = c;
+    }
+
+    return -(i - read);
   }
 }
 
